@@ -68,7 +68,8 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-celt, "
-        "rate = (int) [ 32000, 64000 ], " "channels = (int) [ 1, 2 ]")
+        "rate = (int) [ 32000, 64000 ], "
+        "channels = (int) [ 1, 2 ], " "frame-size = (int) [ 64, 512 ]")
     );
 
 static const GstElementDetails celtenc_details =
@@ -77,14 +78,20 @@ GST_ELEMENT_DETAILS ("Celt audio encoder",
     "Encodes audio in Celt format",
     "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
 
-#define DEFAULT_BITRATE         128
-#define DEFAULT_FRAMESIZE       256
+#define DEFAULT_BITRATE         64
+#define DEFAULT_FRAMESIZE       480
+#define DEFAULT_CBR             TRUE
+#define DEFAULT_COMPLEXITY      9
+#define DEFAULT_MAX_BITRATE     64
 
 enum
 {
   PROP_0,
   PROP_BITRATE,
-  PROP_FRAMESIZE
+  PROP_FRAMESIZE,
+  PROP_CBR,
+  PROP_COMPLEXITY,
+  PROP_MAX_BITRATE
 };
 
 static void gst_celt_enc_finalize (GObject * object);
@@ -106,9 +113,16 @@ static void
 gst_celt_enc_setup_interfaces (GType celtenc_type)
 {
   static const GInterfaceInfo tag_setter_info = { NULL, NULL, NULL };
+  const GInterfaceInfo preset_interface_info = {
+    NULL,                       /* interface_init */
+    NULL,                       /* interface_finalize */
+    NULL                        /* interface_data */
+  };
 
   g_type_add_interface_static (celtenc_type, GST_TYPE_TAG_SETTER,
       &tag_setter_info);
+  g_type_add_interface_static (celtenc_type, GST_TYPE_PRESET,
+      &preset_interface_info);
 
   GST_DEBUG_CATEGORY_INIT (celtenc_debug, "celtenc", 0, "Celt encoder");
 }
@@ -145,11 +159,25 @@ gst_celt_enc_class_init (GstCeltEncClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BITRATE,
       g_param_spec_int ("bitrate", "Encoding Bit-rate",
-          "Specify an encoding bit-rate (in bps). (0 = automatic)",
-          0, 150, DEFAULT_BITRATE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "Specify an encoding bit-rate (in Kbps).",
+          10, 320, DEFAULT_BITRATE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_FRAMESIZE,
       g_param_spec_int ("framesize", "Frame Size",
-          "The number of samples per frame", 64, 256, DEFAULT_FRAMESIZE,
+          "The number of samples per frame", 64, 512, DEFAULT_FRAMESIZE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_CBR,
+      g_param_spec_boolean ("cbr", "Constant bit rate",
+          "Constant bit rate", DEFAULT_CBR,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_COMPLEXITY,
+      g_param_spec_int ("complexity", "Complexity",
+          "Complexity", 0, 10, DEFAULT_COMPLEXITY,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_MAX_BITRATE,
+      g_param_spec_int ("max-bitrate", "Maximum Encoding Bit-rate",
+          "Specify a maximum encoding bit rate (in Kbps) for variable bit rate encoding.",
+          10, 320, DEFAULT_MAX_BITRATE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_celt_enc_finalize);
@@ -175,14 +203,28 @@ gst_celt_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstCeltEnc *enc;
   GstStructure *structure;
+  GstCaps *otherpadcaps;
 
   enc = GST_CELT_ENC (GST_PAD_PARENT (pad));
   enc->setup = FALSE;
+  enc->frame_size = DEFAULT_FRAMESIZE;
+  otherpadcaps = gst_pad_get_allowed_caps (pad);
 
   structure = gst_caps_get_structure (caps, 0);
   gst_structure_get_int (structure, "channels", &enc->channels);
   gst_structure_get_int (structure, "rate", &enc->rate);
 
+  if (otherpadcaps) {
+    if (!gst_caps_is_empty (otherpadcaps)) {
+      GstStructure *ps = gst_caps_get_structure (otherpadcaps, 0);
+      gst_structure_get_int (ps, "frame-size", &enc->frame_size);
+    }
+    gst_caps_unref (otherpadcaps);
+  }
+
+  GST_DEBUG_OBJECT (pad, "channels=%d rate=%d frame-size=%d",
+      enc->channels, enc->rate, enc->frame_size);
+
   gst_celt_enc_setup (enc);
 
   return enc->setup;
@@ -503,6 +545,9 @@ gst_celt_enc_init (GstCeltEnc * enc, GstCeltEncClass * klass)
 
   enc->bitrate = DEFAULT_BITRATE;
   enc->frame_size = DEFAULT_FRAMESIZE;
+  enc->cbr = DEFAULT_CBR;
+  enc->complexity = DEFAULT_COMPLEXITY;
+  enc->max_bitrate = DEFAULT_MAX_BITRATE;
 
   enc->setup = FALSE;
   enc->header_sent = FALSE;
@@ -543,44 +588,43 @@ static gboolean
 gst_celt_enc_setup (GstCeltEnc * enc)
 {
   gint error = CELT_OK;
-  gint bytes_per_packet;
 
   enc->setup = FALSE;
 
-  /* Fix bitrate */
-  if (enc->channels == 1) {
-    if (enc->bitrate <= 0)
-      enc->bitrate = 64;
-    else if (enc->bitrate < 32)
-      enc->bitrate = 32;
-    else if (enc->bitrate > 110)
-      enc->bitrate = 110;
-  } else {
-    if (enc->bitrate <= 0)
-      enc->bitrate = 128;
-    else if (enc->bitrate < 64)
-      enc->bitrate = 64;
-    else if (enc->bitrate > 150)
-      enc->bitrate = 150;
-  }
-
+#ifdef HAVE_CELT_0_7
+  enc->mode = celt_mode_create (enc->rate, enc->frame_size, &error);
+#else
   enc->mode =
       celt_mode_create (enc->rate, enc->channels, enc->frame_size, &error);
+#endif
   if (!enc->mode)
     goto mode_initialization_failed;
 
   celt_mode_info (enc->mode, CELT_GET_FRAME_SIZE, &enc->frame_size);
 
-  bytes_per_packet =
-      (enc->bitrate * 1000 * enc->frame_size / enc->rate + 4) / 8;
-
+#ifdef HAVE_CELT_0_7
+  celt_header_init (&enc->header, enc->mode, enc->channels);
+#else
   celt_header_init (&enc->header, enc->mode);
+#endif
   enc->header.nb_channels = enc->channels;
 
+#ifdef HAVE_CELT_0_7
+  enc->state = celt_encoder_create (enc->mode, enc->channels, &error);
+#else
   enc->state = celt_encoder_create (enc->mode);
+#endif
   if (!enc->state)
     goto encoder_creation_failed;
 
+#ifdef CELT_SET_VBR_RATE
+  if (!enc->cbr) {
+    GST_ERROR ("setting vbr");
+    celt_encoder_ctl (enc->state, CELT_SET_VBR_RATE (enc->bitrate), 0);
+  }
+#endif
+  celt_encoder_ctl (enc->state, CELT_SET_COMPLEXITY (enc->complexity), 0);
+
   GST_LOG_OBJECT (enc, "we have frame size %d", enc->frame_size);
 
   enc->setup = TRUE;
@@ -592,7 +636,11 @@ mode_initialization_failed:
   return FALSE;
 
 encoder_creation_failed:
+#ifdef HAVE_CELT_0_7
+  GST_ERROR_OBJECT (enc, "Encoder creation failed: %d", error);
+#else
   GST_ERROR_OBJECT (enc, "Encoder creation failed");
+#endif
   return FALSE;
 }
 
@@ -709,8 +757,15 @@ gst_celt_enc_encode (GstCeltEnc * enc, gboolean flush)
   GstFlowReturn ret = GST_FLOW_OK;
   gint frame_size = enc->frame_size;
   gint bytes = frame_size * 2 * enc->channels;
-  gint bytes_per_packet =
-      (enc->bitrate * 1000 * enc->frame_size / enc->rate + 4) / 8;
+  gint bytes_per_packet;
+
+  if (enc->cbr) {
+    bytes_per_packet =
+        (enc->bitrate * 1000 * enc->frame_size / enc->rate + 4) / 8;
+  } else {
+    bytes_per_packet =
+        (enc->max_bitrate * 1000 * enc->frame_size / enc->rate + 4) / 8;
+  }
 
   if (flush && gst_adapter_available (enc->adapter) % bytes != 0) {
     guint diff = gst_adapter_available (enc->adapter) % bytes;
@@ -738,9 +793,15 @@ gst_celt_enc_encode (GstCeltEnc * enc, gboolean flush)
 
     GST_DEBUG_OBJECT (enc, "encoding %d samples (%d bytes)", frame_size, bytes);
 
+#ifdef HAVE_CELT_0_5
     outsize =
-        celt_encode (enc->state, data, GST_BUFFER_DATA (outbuf),
-        bytes_per_packet);
+        celt_encode (enc->state, data, NULL,
+        GST_BUFFER_DATA (outbuf), bytes_per_packet);
+#else
+    outsize =
+        celt_encode (enc->state, data,
+        GST_BUFFER_DATA (outbuf), bytes_per_packet);
+#endif
 
     g_free (data);
 
@@ -811,10 +872,13 @@ gst_celt_enc_chain (GstPad * pad, GstBuffer * buf)
 
     gst_caps_set_simple (caps,
         "rate", G_TYPE_INT, enc->rate,
-        "channels", G_TYPE_INT, enc->channels, NULL);
+        "channels", G_TYPE_INT, enc->channels,
+        "frame-size", G_TYPE_INT, enc->frame_size, NULL);
 
     /* negotiate with these caps */
     GST_DEBUG_OBJECT (enc, "here are the caps: %" GST_PTR_FORMAT, caps);
+    GST_LOG_OBJECT (enc, "rate=%d channels=%d frame-size=%d",
+        enc->rate, enc->channels, enc->frame_size);
     gst_pad_set_caps (enc->srcpad, caps);
 
     gst_buffer_set_caps (buf1, caps);
@@ -946,6 +1010,17 @@ gst_celt_enc_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_FRAMESIZE:
       g_value_set_int (value, enc->frame_size);
       break;
+    case PROP_CBR:
+#ifdef CELT_SET_VBR_RATE
+      g_value_set_boolean (value, enc->cbr);
+#endif
+      break;
+    case PROP_COMPLEXITY:
+      g_value_set_int (value, enc->complexity);
+      break;
+    case PROP_MAX_BITRATE:
+      g_value_set_int (value, enc->max_bitrate);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -967,6 +1042,15 @@ gst_celt_enc_set_property (GObject * object, guint prop_id,
     case PROP_FRAMESIZE:
       enc->frame_size = g_value_get_int (value);
       break;
+    case PROP_CBR:
+      enc->cbr = g_value_get_boolean (value);
+      break;
+    case PROP_COMPLEXITY:
+      enc->complexity = g_value_get_int (value);
+      break;
+    case PROP_MAX_BITRATE:
+      enc->max_bitrate = g_value_get_int (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
