@@ -1,5 +1,6 @@
 /* GStreamer FAAC (Free AAC Encoder) plugin
  * Copyright (C) 2003 Ronald Bultje <rbultje@ronald.bitfreak.net>
+ * Copyright (C) 2009 Mark Nauwelaerts <mnauw@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,9 +18,34 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-faac
+ * @seealso: faad
+ *
+ * faac encodes raw audio to AAC (MPEG-4 part 10) streams.
+ *
+ * The #GstFaac:outputformat property determines whether or not the
+ * AAC data needs additional framing provided by a container
+ * (such as Matroska or Quicktime).
+ * This is required for raw data, whereas ADTS formatted AAC already provides
+ * framing and needs no container.
+ *
+ * The #GstFaac:profile property determines the AAC profile, where the default
+ * LC (Low Complexity) profile is most widely used, supported and suitable for
+ * general use. The other profiles are very rarely used and often not supported.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * <programlisting>
+ * gst-launch audiotestsrc wave=sine num-buffers=100 ! audioconvert ! faac ! matroskamux ! filesink location=sine.mkv
+ * </programlisting>
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include <stdlib.h>
 #include <string.h>
 
 #include "gstfaac.h"
@@ -82,6 +108,8 @@ enum
 static void gst_faac_base_init (GstFaacClass * klass);
 static void gst_faac_class_init (GstFaacClass * klass);
 static void gst_faac_init (GstFaac * faac);
+static void gst_faac_finalize (GObject * object);
+static void gst_faac_reset (GstFaac * faac);
 
 static void gst_faac_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
@@ -89,7 +117,9 @@ static void gst_faac_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec);
 
 static gboolean gst_faac_sink_event (GstPad * pad, GstEvent * event);
+static gboolean gst_faac_configure_source_pad (GstFaac * faac);
 static gboolean gst_faac_sink_setcaps (GstPad * pad, GstCaps * caps);
+static GstFlowReturn gst_faac_push_buffers (GstFaac * faac, gboolean force);
 static GstFlowReturn gst_faac_chain (GstPad * pad, GstBuffer * data);
 static GstStateChangeReturn gst_faac_change_state (GstElement * element,
     GstStateChange transition);
@@ -100,6 +130,12 @@ GST_DEBUG_CATEGORY_STATIC (faac_debug);
 #define GST_CAT_DEFAULT faac_debug
 
 #define FAAC_DEFAULT_MPEGVERSION 4
+#define FAAC_DEFAULT_OUTPUTFORMAT 0     /* RAW */
+#define FAAC_DEFAULT_BITRATE      128 * 1000
+#define FAAC_DEFAULT_PROFILE      LOW
+#define FAAC_DEFAULT_TNS          FALSE
+#define FAAC_DEFAULT_MIDSIDE      TRUE
+#define FAAC_DEFAULT_SHORTCTL     SHORTCTL_NORMAL
 
 GType
 gst_faac_get_type (void)
@@ -118,9 +154,17 @@ gst_faac_get_type (void)
       0,
       (GInstanceInitFunc) gst_faac_init,
     };
+    const GInterfaceInfo preset_interface_info = {
+      NULL,                     /* interface_init */
+      NULL,                     /* interface_finalize */
+      NULL                      /* interface_data */
+    };
 
     gst_faac_type = g_type_register_static (GST_TYPE_ELEMENT,
         "GstFaac", &gst_faac_info, 0);
+
+    g_type_add_interface_static (gst_faac_type, GST_TYPE_PRESET,
+        &preset_interface_info);
   }
 
   return gst_faac_type;
@@ -214,28 +258,30 @@ gst_faac_class_init (GstFaacClass * klass)
 
   gobject_class->set_property = gst_faac_set_property;
   gobject_class->get_property = gst_faac_get_property;
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_faac_finalize);
 
   /* properties */
   g_object_class_install_property (gobject_class, ARG_BITRATE,
       g_param_spec_int ("bitrate", "Bitrate (bps)", "Bitrate in bits/sec",
-          8 * 1000, 320 * 1000, 128 * 1000, G_PARAM_READWRITE));
+          8 * 1000, 320 * 1000, FAAC_DEFAULT_BITRATE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_PROFILE,
       g_param_spec_enum ("profile", "Profile", "MPEG/AAC encoding profile",
-          GST_TYPE_FAAC_PROFILE, MAIN, G_PARAM_READWRITE));
+          GST_TYPE_FAAC_PROFILE, FAAC_DEFAULT_PROFILE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_TNS,
       g_param_spec_boolean ("tns", "TNS", "Use temporal noise shaping",
-          FALSE, G_PARAM_READWRITE));
+          FAAC_DEFAULT_TNS, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_MIDSIDE,
       g_param_spec_boolean ("midside", "Midside", "Allow mid/side encoding",
-          TRUE, G_PARAM_READWRITE));
+          FAAC_DEFAULT_MIDSIDE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_SHORTCTL,
       g_param_spec_enum ("shortctl", "Block type",
           "Block type encorcing",
-          GST_TYPE_FAAC_SHORTCTL, MAIN, G_PARAM_READWRITE));
+          GST_TYPE_FAAC_SHORTCTL, FAAC_DEFAULT_SHORTCTL, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_OUTPUTFORMAT,
       g_param_spec_enum ("outputformat", "Output format",
           "Format of output frames",
-          GST_TYPE_FAAC_OUTPUTFORMAT, 0 /* RAW */ , G_PARAM_READWRITE));
+          GST_TYPE_FAAC_OUTPUTFORMAT, FAAC_DEFAULT_OUTPUTFORMAT,
+          G_PARAM_READWRITE));
 
   /* virtual functions */
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_faac_change_state);
@@ -244,14 +290,6 @@ gst_faac_class_init (GstFaacClass * klass)
 static void
 gst_faac_init (GstFaac * faac)
 {
-  faac->handle = NULL;
-  faac->samplerate = -1;
-  faac->channels = -1;
-  faac->cache = NULL;
-  faac->cache_time = GST_CLOCK_TIME_NONE;
-  faac->cache_duration = 0;
-  faac->next_ts = GST_CLOCK_TIME_NONE;
-
   faac->sinkpad = gst_pad_new_from_static_template (&sink_template, "sink");
   gst_pad_set_chain_function (faac->sinkpad,
       GST_DEBUG_FUNCPTR (gst_faac_chain));
@@ -265,13 +303,47 @@ gst_faac_init (GstFaac * faac)
   gst_pad_use_fixed_caps (faac->srcpad);
   gst_element_add_pad (GST_ELEMENT (faac), faac->srcpad);
 
+  faac->adapter = gst_adapter_new ();
+
   /* default properties */
-  faac->bitrate = 1000 * 128;
-  faac->profile = MAIN;
-  faac->shortctl = SHORTCTL_NORMAL;
-  faac->outputformat = 0;       /* RAW */
-  faac->tns = FALSE;
-  faac->midside = TRUE;
+  faac->bitrate = FAAC_DEFAULT_BITRATE;
+  faac->profile = FAAC_DEFAULT_PROFILE;
+  faac->shortctl = FAAC_DEFAULT_SHORTCTL;
+  faac->outputformat = FAAC_DEFAULT_OUTPUTFORMAT;
+  faac->tns = FAAC_DEFAULT_TNS;
+  faac->midside = FAAC_DEFAULT_MIDSIDE;
+
+  gst_faac_reset (faac);
+}
+
+static void
+gst_faac_reset (GstFaac * faac)
+{
+  faac->handle = NULL;
+  faac->samplerate = -1;
+  faac->channels = -1;
+  faac->offset = 0;
+  gst_adapter_clear (faac->adapter);
+}
+
+static void
+gst_faac_finalize (GObject * object)
+{
+  GstFaac *faac = (GstFaac *) object;
+
+  g_object_unref (faac->adapter);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static void
+gst_faac_close_encoder (GstFaac * faac)
+{
+  if (faac->handle)
+    faacEncClose (faac->handle);
+  faac->handle = NULL;
+  gst_adapter_clear (faac->adapter);
+  faac->offset = 0;
 }
 
 static gboolean
@@ -285,27 +357,13 @@ gst_faac_sink_setcaps (GstPad * pad, GstCaps * caps)
   gboolean result = FALSE;
 
   if (!gst_caps_is_fixed (caps))
-    goto done;
-
-  GST_OBJECT_LOCK (faac);
-  if (faac->handle) {
-    faacEncClose (faac->handle);
-    faac->handle = NULL;
-  }
-  if (faac->cache) {
-    gst_buffer_unref (faac->cache);
-    faac->cache = NULL;
-  }
-  GST_OBJECT_UNLOCK (faac);
+    goto refuse_caps;
 
   if (!gst_structure_get_int (structure, "channels", &channels) ||
       !gst_structure_get_int (structure, "rate", &samplerate)) {
-    goto done;
+    goto refuse_caps;
   }
 
-  if (!(handle = faacEncOpen (samplerate, channels, &samples, &bytes)))
-    goto done;
-
   if (gst_structure_has_name (structure, "audio/x-raw-int")) {
     gst_structure_get_int (structure, "width", &width);
     switch (width) {
@@ -326,12 +384,26 @@ gst_faac_sink_setcaps (GstPad * pad, GstCaps * caps)
     bps = 4;
   }
 
-  if (!fmt) {
-    faacEncClose (handle);
-    goto done;
+  if (!fmt)
+    goto refuse_caps;
+
+  /* If the encoder is initialized, do not
+     reinitialize it again if not necessary */
+  if (faac->handle) {
+    if (samplerate == faac->samplerate && channels == faac->channels &&
+        fmt == faac->format)
+      return TRUE;
+
+    /* clear out pending frames */
+    gst_faac_push_buffers (faac, TRUE);
+
+    gst_faac_close_encoder (faac);
   }
 
-  GST_OBJECT_LOCK (faac);
+  if (!(handle = faacEncOpen (samplerate, channels, &samples, &bytes)))
+    goto setup_failed;
+
+  /* ok, record and set up */
   faac->format = fmt;
   faac->bps = bps;
   faac->handle = handle;
@@ -339,13 +411,25 @@ gst_faac_sink_setcaps (GstPad * pad, GstCaps * caps)
   faac->samples = samples;
   faac->channels = channels;
   faac->samplerate = samplerate;
-  GST_OBJECT_UNLOCK (faac);
 
-  result = TRUE;
+  /* finish up */
+  result = gst_faac_configure_source_pad (faac);
 
 done:
   gst_object_unref (faac);
   return result;
+
+  /* ERRORS */
+setup_failed:
+  {
+    GST_ELEMENT_ERROR (faac, LIBRARY, SETTINGS, (NULL), (NULL));
+    goto done;
+  }
+refuse_caps:
+  {
+    GST_WARNING_OBJECT (faac, "refused caps %" GST_PTR_FORMAT, caps);
+    goto done;
+  }
 }
 
 static gboolean
@@ -354,7 +438,7 @@ gst_faac_configure_source_pad (GstFaac * faac)
   GstCaps *allowed_caps;
   GstCaps *srccaps;
   gboolean ret = FALSE;
-  gint n, ver, mpegversion;
+  gint n, ver, mpegversion = 2;
   faacEncConfiguration *conf;
   guint maxbitrate;
 
@@ -363,24 +447,23 @@ gst_faac_configure_source_pad (GstFaac * faac)
   allowed_caps = gst_pad_get_allowed_caps (faac->srcpad);
   GST_DEBUG_OBJECT (faac, "allowed caps: %" GST_PTR_FORMAT, allowed_caps);
 
-  if (allowed_caps == NULL)
-    return FALSE;
+  if (allowed_caps) {
+    if (gst_caps_is_empty (allowed_caps))
+      goto empty_caps;
 
-  if (gst_caps_is_empty (allowed_caps))
-    goto empty_caps;
+    if (!gst_caps_is_any (allowed_caps)) {
+      for (n = 0; n < gst_caps_get_size (allowed_caps); n++) {
+        GstStructure *s = gst_caps_get_structure (allowed_caps, n);
 
-  if (!gst_caps_is_any (allowed_caps)) {
-    for (n = 0; n < gst_caps_get_size (allowed_caps); n++) {
-      GstStructure *s = gst_caps_get_structure (allowed_caps, n);
-
-      if (gst_structure_get_int (s, "mpegversion", &ver) &&
-          (ver == 4 || ver == 2)) {
-        mpegversion = ver;
-        break;
+        if (gst_structure_get_int (s, "mpegversion", &ver) &&
+            (ver == 4 || ver == 2)) {
+          mpegversion = ver;
+          break;
+        }
       }
     }
+    gst_caps_unref (allowed_caps);
   }
-  gst_caps_unref (allowed_caps);
 
   /* we negotiated caps update current configuration */
   conf = faacEncGetCurrentConfiguration (faac->handle);
@@ -428,6 +511,7 @@ gst_faac_configure_source_pad (GstFaac * faac)
     /* copy it into a buffer */
     codec_data = gst_buffer_new_and_alloc (config_len);
     memcpy (GST_BUFFER_DATA (codec_data), config, config_len);
+    free (config);
 
     /* add to caps */
     gst_caps_set_simple (srccaps,
@@ -456,6 +540,138 @@ set_failed:
   }
 }
 
+static GstFlowReturn
+gst_faac_push_buffers (GstFaac * faac, gboolean force)
+{
+  GstFlowReturn ret = GST_FLOW_OK;
+  gint av, frame_size, size, ret_size;
+  GstBuffer *outbuf;
+  guint64 timestamp, distance;
+  const guint8 *data;
+
+  /* samples already considers channel count */
+  frame_size = faac->samples * faac->bps;
+
+  while (G_LIKELY (ret == GST_FLOW_OK)) {
+
+    av = gst_adapter_available (faac->adapter);
+
+    GST_LOG_OBJECT (faac, "pushing: force: %d, frame_size: %d, av: %d, "
+        "offset: %d", force, frame_size, av, faac->offset);
+
+    /* idea:
+     * - start of adapter corresponds with what has already been encoded
+     * (i.e. really returned by faac)
+     * - start + offset is what needs to be fed to faac next
+     * That way we can timestamp the output based
+     * on adapter provided timestamp (and duration is a fixed frame duration) */
+
+    /* not enough data for one frame and no flush forcing */
+    if (!force && (av < frame_size + faac->offset))
+      break;
+
+    if (G_LIKELY (av - faac->offset >= frame_size)) {
+      GST_LOG_OBJECT (faac, "encoding a frame");
+      data = gst_adapter_peek (faac->adapter, faac->offset + frame_size);
+      data += faac->offset;
+      size = frame_size;
+    } else if (av - faac->offset > 0) {
+      GST_LOG_OBJECT (faac, "encoding leftover");
+      data = gst_adapter_peek (faac->adapter, av);
+      data += faac->offset;
+      size = av - faac->offset;
+    } else {
+      GST_LOG_OBJECT (faac, "emptying encoder");
+      data = NULL;
+      size = 0;
+    }
+
+    outbuf = gst_buffer_new_and_alloc (faac->bytes);
+
+    if (G_UNLIKELY ((ret_size = faacEncEncode (faac->handle, (gint32 *) data,
+                    size / faac->bps, GST_BUFFER_DATA (outbuf),
+                    faac->bytes)) < 0)) {
+      gst_buffer_unref (outbuf);
+      goto encode_failed;
+    }
+
+    GST_LOG_OBJECT (faac, "encoder return: %d", ret_size);
+
+    /* consumed, advanced view */
+    faac->offset += size;
+    g_assert (faac->offset <= av);
+
+    if (G_UNLIKELY (!ret_size)) {
+      gst_buffer_unref (outbuf);
+      if (size)
+        continue;
+      else
+        break;
+    }
+
+    /* in case encoder returns more data than is expected (which seems possible)
+     * ignore the extra part */
+    if (G_UNLIKELY (av == 0 && faac->offset == 0)) {
+      GST_DEBUG_OBJECT (faac, "encoder returned unexpected data; discarding");
+      gst_buffer_unref (outbuf);
+      continue;
+    }
+
+    /* after some caching, finally some data */
+    /* adapter gives time */
+    timestamp = gst_adapter_prev_timestamp (faac->adapter, &distance);
+    if (G_LIKELY ((av = gst_adapter_available (faac->adapter)) >= frame_size)) {
+      /* must have then come from a complete frame */
+      gst_adapter_flush (faac->adapter, frame_size);
+      faac->offset -= frame_size;
+      size = frame_size;
+    } else {
+      /* otherwise leftover */
+      gst_adapter_clear (faac->adapter);
+      faac->offset = 0;
+      size = av;
+    }
+
+    GST_BUFFER_SIZE (outbuf) = ret_size;
+    if (G_LIKELY (GST_CLOCK_TIME_IS_VALID (timestamp)))
+      GST_BUFFER_TIMESTAMP (outbuf) = timestamp +
+          GST_FRAMES_TO_CLOCK_TIME (distance / faac->channels / faac->bps,
+          faac->samplerate);
+    GST_BUFFER_DURATION (outbuf) =
+        GST_FRAMES_TO_CLOCK_TIME (size / faac->channels / faac->bps,
+        faac->samplerate);
+    /* perhaps check/set DISCONT based on timestamps ? */
+
+    GST_LOG_OBJECT (faac, "Pushing out buffer time: %" GST_TIME_FORMAT
+        " duration: %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
+        GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
+
+    gst_buffer_set_caps (outbuf, GST_PAD_CAPS (faac->srcpad));
+    ret = gst_pad_push (faac->srcpad, outbuf);
+  }
+
+  /* in case encoder returns less than expected, clear our view as well */
+  if (G_UNLIKELY (force)) {
+#ifndef GST_DISABLE_GST_DEBUG
+    if ((av = gst_adapter_available (faac->adapter)))
+      GST_WARNING_OBJECT (faac, "encoder left %d bytes; discarding", av);
+#endif
+    gst_adapter_clear (faac->adapter);
+    faac->offset = 0;
+  }
+
+  return ret;
+
+  /* ERRORS */
+encode_failed:
+  {
+    GST_ELEMENT_ERROR (faac, LIBRARY, ENCODE, (NULL), (NULL));
+    gst_buffer_unref (outbuf);
+    return GST_FLOW_ERROR;
+  }
+}
+
 static gboolean
 gst_faac_sink_event (GstPad * pad, GstEvent * event)
 {
@@ -464,55 +680,20 @@ gst_faac_sink_event (GstPad * pad, GstEvent * event)
 
   faac = GST_FAAC (gst_pad_get_parent (pad));
 
+  GST_LOG_OBJECT (faac, "received %s", GST_EVENT_TYPE_NAME (event));
+
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
     {
-      GstBuffer *outbuf;
-
-      if (!faac->handle)
-        ret = FALSE;
-      else
-        ret = TRUE;
-
-      /* flush first */
-      GST_DEBUG ("Pushing out remaining buffers because of EOS");
-      while (ret) {
-        if (gst_pad_alloc_buffer_and_set_caps (faac->srcpad,
-                GST_BUFFER_OFFSET_NONE, faac->bytes,
-                GST_PAD_CAPS (faac->srcpad), &outbuf) == GST_FLOW_OK) {
-          gint ret_size;
-
-          GST_DEBUG ("next_ts %" GST_TIME_FORMAT,
-              GST_TIME_ARGS (faac->next_ts));
-
-          if ((ret_size = faacEncEncode (faac->handle, NULL, 0,
-                      GST_BUFFER_DATA (outbuf), faac->bytes)) > 0) {
-            GST_BUFFER_SIZE (outbuf) = ret_size;
-            GST_BUFFER_TIMESTAMP (outbuf) = faac->next_ts;
-            /* faac seems to always consume a fixed number of input samples,
-             * therefore extrapolate the duration from that value and the incoming
-             * bitrate */
-            GST_BUFFER_DURATION (outbuf) = gst_util_uint64_scale (faac->samples,
-                GST_SECOND, faac->channels * faac->samplerate);
-            if (GST_CLOCK_TIME_IS_VALID (faac->next_ts))
-              faac->next_ts += GST_BUFFER_DURATION (outbuf);
-            gst_pad_push (faac->srcpad, outbuf);
-          } else {
-            gst_buffer_unref (outbuf);
-            ret = FALSE;
-          }
-        } else
-          ret = FALSE;
+      if (faac->handle) {
+        /* flush first */
+        GST_DEBUG_OBJECT (faac, "Pushing out remaining buffers because of EOS");
+        gst_faac_push_buffers (faac, TRUE);
       }
+
       ret = gst_pad_event_default (pad, event);
       break;
     }
-    case GST_EVENT_NEWSEGMENT:
-      ret = gst_pad_push_event (faac->srcpad, event);
-      break;
-    case GST_EVENT_TAG:
-      ret = gst_pad_event_default (pad, event);
-      break;
     default:
       ret = gst_pad_event_default (pad, event);
       break;
@@ -526,138 +707,20 @@ static GstFlowReturn
 gst_faac_chain (GstPad * pad, GstBuffer * inbuf)
 {
   GstFlowReturn result = GST_FLOW_OK;
-  GstBuffer *outbuf, *subbuf;
   GstFaac *faac;
-  guint size, ret_size, in_size, frame_size;
 
   faac = GST_FAAC (gst_pad_get_parent (pad));
 
   if (!faac->handle)
     goto no_handle;
 
-  if (!GST_PAD_CAPS (faac->srcpad)) {
-    if (!gst_faac_configure_source_pad (faac))
-      goto nego_failed;
-  }
-
-  GST_DEBUG ("Got buffer time:%" GST_TIME_FORMAT " duration:%" GST_TIME_FORMAT,
-      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (inbuf)),
+  GST_LOG_OBJECT (faac, "Got buffer time: %" GST_TIME_FORMAT " duration: %"
+      GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (inbuf)),
       GST_TIME_ARGS (GST_BUFFER_DURATION (inbuf)));
 
-  size = GST_BUFFER_SIZE (inbuf);
-  in_size = size;
-  if (faac->cache)
-    in_size += GST_BUFFER_SIZE (faac->cache);
-  frame_size = faac->samples * faac->bps;
+  gst_adapter_push (faac->adapter, inbuf);
 
-  while (1) {
-    /* do we have enough data for one frame? */
-    if (in_size / faac->bps < faac->samples) {
-      if (in_size > size) {
-        GstBuffer *merge;
-
-        /* this is panic! we got a buffer, but still don't have enough
-         * data. Merge them and retry in the next cycle... */
-        merge = gst_buffer_merge (faac->cache, inbuf);
-        gst_buffer_unref (faac->cache);
-        gst_buffer_unref (inbuf);
-        faac->cache = merge;
-      } else if (in_size == size) {
-        /* this shouldn't happen, but still... */
-        faac->cache = inbuf;
-      } else if (in_size > 0) {
-        faac->cache = gst_buffer_create_sub (inbuf, size - in_size, in_size);
-        GST_BUFFER_DURATION (faac->cache) =
-            GST_BUFFER_DURATION (inbuf) * GST_BUFFER_SIZE (faac->cache) / size;
-        GST_BUFFER_TIMESTAMP (faac->cache) =
-            GST_BUFFER_TIMESTAMP (inbuf) + (GST_BUFFER_DURATION (inbuf) *
-            (size - in_size) / size);
-        gst_buffer_unref (inbuf);
-      } else {
-        gst_buffer_unref (inbuf);
-      }
-
-      goto done;
-    }
-
-    /* create the frame */
-    if (in_size > size) {
-      GstBuffer *merge;
-
-      /* merge */
-      subbuf = gst_buffer_create_sub (inbuf, 0, frame_size - (in_size - size));
-      GST_BUFFER_DURATION (subbuf) =
-          GST_BUFFER_DURATION (inbuf) * GST_BUFFER_SIZE (subbuf) / size;
-      merge = gst_buffer_merge (faac->cache, subbuf);
-      gst_buffer_unref (faac->cache);
-      gst_buffer_unref (subbuf);
-      subbuf = merge;
-      faac->cache = NULL;
-    } else {
-      subbuf = gst_buffer_create_sub (inbuf, size - in_size, frame_size);
-      GST_BUFFER_DURATION (subbuf) =
-          GST_BUFFER_DURATION (inbuf) * GST_BUFFER_SIZE (subbuf) / size;
-      GST_BUFFER_TIMESTAMP (subbuf) =
-          GST_BUFFER_TIMESTAMP (inbuf) + (GST_BUFFER_DURATION (inbuf) *
-          (size - in_size) / size);
-    }
-
-    result =
-        gst_pad_alloc_buffer_and_set_caps (faac->srcpad, GST_BUFFER_OFFSET_NONE,
-        faac->bytes, GST_PAD_CAPS (faac->srcpad), &outbuf);
-    if (result != GST_FLOW_OK)
-      goto done;
-
-    if ((ret_size = faacEncEncode (faac->handle,
-                (gint32 *) GST_BUFFER_DATA (subbuf),
-                GST_BUFFER_SIZE (subbuf) / faac->bps,
-                GST_BUFFER_DATA (outbuf), faac->bytes)) < 0) {
-      GST_ELEMENT_ERROR (faac, LIBRARY, ENCODE, (NULL), (NULL));
-      gst_buffer_unref (inbuf);
-      gst_buffer_unref (subbuf);
-      result = GST_FLOW_ERROR;
-      goto done;
-    }
-
-    if (ret_size > 0) {
-      GST_BUFFER_SIZE (outbuf) = ret_size;
-      if (faac->cache_time != GST_CLOCK_TIME_NONE) {
-        GST_BUFFER_TIMESTAMP (outbuf) = faac->cache_time;
-        faac->cache_time = GST_CLOCK_TIME_NONE;
-      } else
-        GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (subbuf);
-      GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (subbuf);
-      if (faac->cache_duration) {
-        GST_BUFFER_DURATION (outbuf) += faac->cache_duration;
-        faac->cache_duration = 0;
-      }
-      /* Store the value of the next expected timestamp to output
-       * This is required in order to output the trailing encoded packets
-       * at EOS with proper timestamps and duration. */
-      faac->next_ts =
-          GST_BUFFER_TIMESTAMP (outbuf) + GST_BUFFER_DURATION (outbuf);
-      GST_DEBUG ("Pushing out buffer time:%" GST_TIME_FORMAT " duration:%"
-          GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
-          GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
-      result = gst_pad_push (faac->srcpad, outbuf);
-    } else {
-      /* FIXME: what I'm doing here isn't fully correct, but there
-       * really isn't a better way yet.
-       * Problem is that libfaac caches buffers (for encoding
-       * purposes), so the timestamp of the outgoing buffer isn't
-       * the same as the timestamp of the data that I pushed in.
-       * However, I don't know the delay between those two so I
-       * cannot really say aything about it. This is a bad guess. */
-
-      gst_buffer_unref (outbuf);
-      if (faac->cache_time != GST_CLOCK_TIME_NONE)
-        faac->cache_time = GST_BUFFER_TIMESTAMP (subbuf);
-      faac->cache_duration += GST_BUFFER_DURATION (subbuf);
-    }
-
-    in_size -= frame_size;
-    gst_buffer_unref (subbuf);
-  }
+  result = gst_faac_push_buffers (faac, FALSE);
 
 done:
   gst_object_unref (faac);
@@ -673,14 +736,6 @@ no_handle:
     result = GST_FLOW_ERROR;
     goto done;
   }
-nego_failed:
-  {
-    GST_ELEMENT_ERROR (faac, CORE, NEGOTIATION, (NULL),
-        ("failed to negotiate MPEG/AAC format with next element"));
-    gst_buffer_unref (inbuf);
-    result = GST_FLOW_ERROR;
-    goto done;
-  }
 }
 
 static void
@@ -771,21 +826,8 @@ gst_faac_change_state (GstElement * element, GstStateChange transition)
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
     {
-      GST_OBJECT_LOCK (faac);
-      if (faac->handle) {
-        faacEncClose (faac->handle);
-        faac->handle = NULL;
-      }
-      if (faac->cache) {
-        gst_buffer_unref (faac->cache);
-        faac->cache = NULL;
-      }
-      faac->cache_time = GST_CLOCK_TIME_NONE;
-      faac->cache_duration = 0;
-      faac->samplerate = -1;
-      faac->channels = -1;
-      faac->next_ts = GST_CLOCK_TIME_NONE;
-      GST_OBJECT_UNLOCK (faac);
+      gst_faac_close_encoder (faac);
+      gst_faac_reset (faac);
       break;
     }
     default:
@@ -798,7 +840,8 @@ gst_faac_change_state (GstElement * element, GstStateChange transition)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  return gst_element_register (plugin, "faac", GST_RANK_NONE, GST_TYPE_FAAC);
+  return gst_element_register (plugin, "faac", GST_RANK_SECONDARY,
+      GST_TYPE_FAAC);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
