@@ -24,7 +24,7 @@
  * 
  * The ladspa (Linux Audio Developer's Simple Plugin API) element is a bridge
  * for plugins using the <ulink url="http://www.ladspa.org/">ladspa</ulink> API.
- * It scans all installed ladspa plugins and registeres them as gstreamer
+ * It scans all installed ladspa plugins and registers them as gstreamer
  * elements. If available it can also parse lrdf files and use the metadata for
  * element classification.
  */
@@ -125,10 +125,10 @@ gst_ladspa_base_init (gpointer g_class)
 
       if (LADSPA_IS_PORT_INPUT (p))
         gst_signal_processor_class_add_pad_template (gsp_class, name,
-            GST_PAD_SINK, gsp_class->num_audio_in++);
+            GST_PAD_SINK, gsp_class->num_audio_in++, 1);
       else
         gst_signal_processor_class_add_pad_template (gsp_class, name,
-            GST_PAD_SRC, gsp_class->num_audio_out++);
+            GST_PAD_SRC, gsp_class->num_audio_out++, 1);
 
       g_free (name);
     } else if (LADSPA_IS_PORT_CONTROL (p)) {
@@ -313,14 +313,17 @@ gst_ladspa_class_get_param_name (GstLADSPAClass * klass, gint portnum)
     gint n = 1;
     gchar *nret = g_strdup_printf ("%s-%d", ret, n++);
 
-    g_free (ret);
     while (g_object_class_find_property (G_OBJECT_CLASS (klass), nret)) {
       g_free (nret);
       nret = g_strdup_printf ("%s-%d", ret, n++);
     }
+    g_free (ret);
     ret = nret;
   }
 
+  GST_DEBUG ("built property name '%s' from port name '%s'", ret,
+      desc->PortNames[portnum]);
+
   return ret;
 }
 
