@@ -24,6 +24,7 @@
 #endif
 
 #include <gst/gst.h>
+#include <stdio.h>
 #include <string.h>
 #include "gstmms.h"
 
@@ -60,8 +61,7 @@ static void gst_mms_set_property (GObject * object, guint prop_id,
 static void gst_mms_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-static const GstQueryType *gst_mms_get_query_types (GstPad * pad);
-static gboolean gst_mms_src_query (GstPad * pad, GstQuery * query);
+static gboolean gst_mms_query (GstBaseSrc * src, GstQuery * query);
 
 static gboolean gst_mms_start (GstBaseSrc * bsrc);
 static gboolean gst_mms_stop (GstBaseSrc * bsrc);
@@ -138,6 +138,7 @@ gst_mms_class_init (GstMMSClass * klass)
   gstbasesrc_class->prepare_seek_segment =
       GST_DEBUG_FUNCPTR (gst_mms_prepare_seek_segment);
   gstbasesrc_class->do_seek = GST_DEBUG_FUNCPTR (gst_mms_do_seek);
+  gstbasesrc_class->query = GST_DEBUG_FUNCPTR (gst_mms_query);
 }
 
 /* initialize the new element
@@ -148,11 +149,6 @@ gst_mms_class_init (GstMMSClass * klass)
 static void
 gst_mms_init (GstMMS * mmssrc, GstMMSClass * g_class)
 {
-  gst_pad_set_query_function (GST_BASE_SRC (mmssrc)->srcpad,
-      GST_DEBUG_FUNCPTR (gst_mms_src_query));
-  gst_pad_set_query_type_function (GST_BASE_SRC (mmssrc)->srcpad,
-      GST_DEBUG_FUNCPTR (gst_mms_get_query_types));
-
   mmssrc->uri_name = NULL;
   mmssrc->current_connection_uri_name = NULL;
   mmssrc->connection = NULL;
@@ -186,27 +182,12 @@ gst_mms_finalize (GObject * gobject)
 
 }
 
-/*
- * location querying and so on.
- */
-
-static const GstQueryType *
-gst_mms_get_query_types (GstPad * pad)
-{
-  static const GstQueryType types[] = {
-    GST_QUERY_POSITION,
-    GST_QUERY_DURATION,
-    0
-  };
-
-  return types;
-}
-
+/* FIXME operating in TIME rather than BYTES could remove this altogether
+ * and be more convenient elsewhere */
 static gboolean
-gst_mms_src_query (GstPad * pad, GstQuery * query)
+gst_mms_query (GstBaseSrc * src, GstQuery * query)
 {
-
-  GstMMS *mmssrc = GST_MMS (gst_pad_get_parent (pad));
+  GstMMS *mmssrc = GST_MMS (src);
   gboolean res = TRUE;
   GstFormat format;
   gint64 value;
@@ -241,13 +222,13 @@ gst_mms_src_query (GstPad * pad, GstQuery * query)
       }
       break;
     default:
-      res = FALSE;
+      /* chain to parent */
+      res =
+          GST_BASE_SRC_CLASS (parent_class)->query (GST_BASE_SRC (src), query);
       break;
   }
 
-  gst_object_unref (mmssrc);
   return res;
-
 }
 
 
@@ -260,23 +241,22 @@ gst_mms_prepare_seek_segment (GstBaseSrc * src, GstEvent * event,
   GstSeekFlags flags;
   GstFormat seek_format;
   gdouble rate;
-  GstMMS *mmssrc = GST_MMS (src);
 
   gst_event_parse_seek (event, &rate, &seek_format, &flags,
       &cur_type, &cur, &stop_type, &stop);
 
   if (seek_format != GST_FORMAT_BYTES && seek_format != GST_FORMAT_TIME) {
-    GST_LOG_OBJECT (mmssrc, "Only byte or time seeking is supported");
+    GST_LOG_OBJECT (src, "Only byte or time seeking is supported");
     return FALSE;
   }
 
   if (stop_type != GST_SEEK_TYPE_NONE) {
-    GST_LOG_OBJECT (mmssrc, "Stop seeking not supported");
+    GST_LOG_OBJECT (src, "Stop seeking not supported");
     return FALSE;
   }
 
   if (cur_type != GST_SEEK_TYPE_NONE && cur_type != GST_SEEK_TYPE_SET) {
-    GST_LOG_OBJECT (mmssrc, "Only absolute seeking is supported");
+    GST_LOG_OBJECT (src, "Only absolute seeking is supported");
     return FALSE;
   }
 
@@ -308,8 +288,8 @@ gst_mms_do_seek (GstBaseSrc * src, GstSegment * segment)
       return FALSE;
     }
     start = mmsx_get_current_pos (mmssrc->connection);
-    GST_LOG_OBJECT (mmssrc, "sought to %f sec, offset after seek: %lld\n",
-        (double) segment->start / GST_SECOND, start);
+    GST_INFO_OBJECT (mmssrc, "sought to %" GST_TIME_FORMAT ", offset after "
+        "seek: %" G_GINT64_FORMAT, GST_TIME_ARGS (segment->start), start);
   } else if (segment->format == GST_FORMAT_BYTES) {
     start = mmsx_seek (NULL, mmssrc->connection, segment->start, SEEK_SET);
     /* mmsx_seek will close and reopen the connection when seeking with the
@@ -318,11 +298,11 @@ gst_mms_do_seek (GstBaseSrc * src, GstSegment * segment)
       GST_DEBUG_OBJECT (mmssrc, "connection broken during seek");
       return FALSE;
     }
-    GST_DEBUG_OBJECT (mmssrc, "sought to: %llu bytes, result: %lld",
-        segment->start, start);
+    GST_INFO_OBJECT (mmssrc, "sought to: %" G_GINT64_FORMAT " bytes, "
+        "result: %" G_GINT64_FORMAT, segment->start, start);
   } else {
-    GST_DEBUG_OBJECT (mmssrc, "unsupported seek segment format: %d",
-        (int) segment->format);
+    GST_DEBUG_OBJECT (mmssrc, "unsupported seek segment format: %s",
+        GST_STR_NULL (gst_format_get_name (segment->format)));
     return FALSE;
   }
   gst_segment_init (segment, GST_FORMAT_BYTES);
@@ -460,8 +440,18 @@ gst_mms_start (GstBaseSrc * bsrc)
     GST_DEBUG_OBJECT (mms, "Connect successful");
     return TRUE;
   } else {
-    GST_ELEMENT_ERROR (mms, RESOURCE, OPEN_READ,
-        ("Could not connect to this stream"), (NULL));
+    gchar *url, *location;
+
+    GST_ERROR_OBJECT (mms,
+        "Could not connect to this stream, redirecting to rtsp");
+    location = gst_uri_get_location (mms->uri_name);
+    url = g_strdup_printf ("rtsp://%s", location);
+    g_free (location);
+
+    gst_element_post_message (GST_ELEMENT_CAST (mms),
+        gst_message_new_element (GST_OBJECT_CAST (mms),
+            gst_structure_new ("redirect", "new-location", G_TYPE_STRING, url,
+                NULL)));
     return FALSE;
   }
 
