@@ -63,7 +63,9 @@ enum
 
 static gboolean rsn_stream_selector_is_active_sinkpad (RsnStreamSelector * sel,
     GstPad * pad);
-static GstPad *rsn_stream_selector_activate_sinkpad (RsnStreamSelector * sel,
+static GstPad *rsn_stream_selector_get_active (RsnStreamSelector * sel,
+    GstPad * pad);
+static void rsn_stream_selector_set_active (RsnStreamSelector * sel,
     GstPad * pad);
 static GstPad *rsn_stream_selector_get_linked_pad (GstPad * pad,
     gboolean strict);
@@ -90,7 +92,6 @@ struct _RsnSelectorPad
 
   gboolean active;
   gboolean eos;
-  gboolean segment_pending;
   GstSegment segment;
   GstTagList *tags;
 };
@@ -112,7 +113,6 @@ static GstPadClass *selector_pad_parent_class = NULL;
 static void gst_selector_pad_reset (RsnSelectorPad * pad);
 static gboolean gst_selector_pad_event (GstPad * pad, GstEvent * event);
 static GstCaps *gst_selector_pad_getcaps (GstPad * pad);
-static GList *gst_selector_pad_get_linked_pads (GstPad * pad);
 static GstFlowReturn gst_selector_pad_chain (GstPad * pad, GstBuffer * buf);
 static GstFlowReturn gst_selector_pad_bufferalloc (GstPad * pad,
     guint64 offset, guint size, GstCaps * caps, GstBuffer ** buf);
@@ -231,23 +231,6 @@ gst_selector_pad_reset (RsnSelectorPad * pad)
   gst_segment_init (&pad->segment, GST_FORMAT_UNDEFINED);
 }
 
-/* strictly get the linked pad from the sinkpad. If the pad is active we return
- * the srcpad else we return NULL */
-static GList *
-gst_selector_pad_get_linked_pads (GstPad * pad)
-{
-  GstPad *otherpad;
-
-  otherpad = rsn_stream_selector_get_linked_pad (pad, TRUE);
-  if (!otherpad)
-    return NULL;
-
-  /* need to drop the ref, internal linked pads is not MT safe */
-  gst_object_unref (otherpad);
-
-  return g_list_append (NULL, otherpad);
-}
-
 static gboolean
 gst_selector_pad_event (GstPad * pad, GstEvent * event)
 {
@@ -261,7 +244,7 @@ gst_selector_pad_event (GstPad * pad, GstEvent * event)
   selpad = GST_SELECTOR_PAD_CAST (pad);
 
   /* only forward if we are dealing with the active sinkpad */
-  active_sinkpad = rsn_stream_selector_activate_sinkpad (sel, pad);
+  active_sinkpad = rsn_stream_selector_get_active (sel, pad);
   forward = (active_sinkpad == pad);
 
   switch (GST_EVENT_TYPE (event)) {
@@ -286,10 +269,6 @@ gst_selector_pad_event (GstPad * pad, GstEvent * event)
 
       gst_segment_set_newsegment_full (&selpad->segment, update,
           rate, arate, format, start, stop, time);
-      /* if we are not going to forward the segment, mark the segment as
-       * pending */
-      if (!forward)
-        selpad->segment_pending = TRUE;
       break;
     }
     case GST_EVENT_TAG:
@@ -307,6 +286,18 @@ gst_selector_pad_event (GstPad * pad, GstEvent * event)
       GST_OBJECT_UNLOCK (selpad);
       break;
     }
+    case GST_EVENT_CUSTOM_DOWNSTREAM:
+    {
+      const GstStructure *structure = gst_event_get_structure (event);
+      if (structure != NULL &&
+          gst_structure_has_name (structure, "application/x-gst-dvd")) {
+        const char *type = gst_structure_get_string (structure, "event");
+        if (strcmp (type, "select-pad") == 0) {
+          rsn_stream_selector_set_active (sel, pad);
+          forward = FALSE;
+        }
+      }
+    }
     case GST_EVENT_EOS:
       selpad->eos = TRUE;
       break;
@@ -351,7 +342,7 @@ gst_selector_pad_bufferalloc (GstPad * pad, guint64 offset,
 
   sel = RSN_STREAM_SELECTOR (gst_pad_get_parent (pad));
 
-  active_sinkpad = rsn_stream_selector_activate_sinkpad (sel, pad);
+  active_sinkpad = rsn_stream_selector_get_active (sel, pad);
 
   /* Fallback allocation for buffers from pads except the selected one */
   if (pad != active_sinkpad) {
@@ -392,12 +383,13 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   RsnSelectorPad *selpad;
   GstClockTime timestamp;
   GstSegment *seg;
+  gboolean discont;
 
   sel = RSN_STREAM_SELECTOR (gst_pad_get_parent (pad));
   selpad = GST_SELECTOR_PAD_CAST (pad);
   seg = &selpad->segment;
 
-  active_sinkpad = rsn_stream_selector_activate_sinkpad (sel, pad);
+  active_sinkpad = rsn_stream_selector_get_active (sel, pad);
 
   timestamp = GST_BUFFER_TIMESTAMP (buf);
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
@@ -410,17 +402,20 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   if (pad != active_sinkpad)
     goto ignore;
 
-  /* if we have a pending segment, push it out now */
-  if (selpad->segment_pending) {
-    gst_pad_push_event (sel->srcpad, gst_event_new_new_segment_full (FALSE,
-            seg->rate, seg->applied_rate, seg->format, seg->start, seg->stop,
-            seg->time));
+  /* If we just switched pads, mark a discont buffer */
+  GST_OBJECT_LOCK (sel);
+  discont = sel->mark_discont;
+  sel->mark_discont = FALSE;
+  GST_OBJECT_UNLOCK (sel);
 
-    selpad->segment_pending = FALSE;
+  if (discont) {
+    GST_DEBUG_OBJECT (sel, "Marking buffer discont due to pad switch");
+    buf = gst_buffer_make_metadata_writable (buf);
+    GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
   }
 
   /* forward */
-  GST_DEBUG_OBJECT (sel, "Forwarding buffer %p from pad %s:%s", buf,
+  GST_LOG_OBJECT (sel, "Forwarding buffer %p from pad %s:%s", buf,
       GST_DEBUG_PAD_NAME (pad));
   res = gst_pad_push (sel->srcpad, buf);
 done:
@@ -453,7 +448,7 @@ static GstPad *rsn_stream_selector_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * unused);
 static void rsn_stream_selector_release_pad (GstElement * element,
     GstPad * pad);
-static GList *rsn_stream_selector_get_linked_pads (GstPad * pad);
+static GstIterator *rsn_stream_selector_iterate_linked_pads (GstPad * pad);
 static GstCaps *rsn_stream_selector_getcaps (GstPad * pad);
 
 static GstElementClass *parent_class = NULL;
@@ -530,8 +525,8 @@ static void
 rsn_stream_selector_init (RsnStreamSelector * sel)
 {
   sel->srcpad = gst_pad_new ("src", GST_PAD_SRC);
-  gst_pad_set_internal_link_function (sel->srcpad,
-      GST_DEBUG_FUNCPTR (rsn_stream_selector_get_linked_pads));
+  gst_pad_set_iterate_internal_links_function (sel->srcpad,
+      GST_DEBUG_FUNCPTR (rsn_stream_selector_iterate_linked_pads));
   gst_pad_set_getcaps_function (sel->srcpad,
       GST_DEBUG_FUNCPTR (rsn_stream_selector_getcaps));
   gst_element_add_pad (GST_ELEMENT (sel), sel->srcpad);
@@ -574,28 +569,10 @@ rsn_stream_selector_set_property (GObject * object, guint prop_id,
     case PROP_ACTIVE_PAD:
     {
       GstPad *pad = NULL;
-      GstPad **active_pad_p;
 
       pad = g_value_get_object (value);
+      rsn_stream_selector_set_active (sel, pad);
 
-      GST_OBJECT_LOCK (object);
-      if (pad != sel->active_sinkpad) {
-        RsnSelectorPad *selpad;
-
-        selpad = GST_SELECTOR_PAD_CAST (pad);
-        /* we can only activate pads that have data received */
-        if (selpad && !selpad->active) {
-          GST_DEBUG_OBJECT (sel, "No data received on pad %" GST_PTR_FORMAT,
-              pad);
-        } else {
-          active_pad_p = &sel->active_sinkpad;
-          gst_object_replace ((GstObject **) active_pad_p,
-              GST_OBJECT_CAST (pad));
-          GST_DEBUG_OBJECT (sel, "New active pad is %" GST_PTR_FORMAT,
-              sel->active_sinkpad);
-        }
-      }
-      GST_OBJECT_UNLOCK (object);
       break;
     }
     default:
@@ -694,7 +671,7 @@ rsn_stream_selector_is_active_sinkpad (RsnStreamSelector * sel, GstPad * pad)
 
 /* Get or create the active sinkpad */
 static GstPad *
-rsn_stream_selector_activate_sinkpad (RsnStreamSelector * sel, GstPad * pad)
+rsn_stream_selector_get_active (RsnStreamSelector * sel, GstPad * pad)
 {
   GstPad *active_sinkpad;
   RsnSelectorPad *selpad;
@@ -714,17 +691,44 @@ rsn_stream_selector_activate_sinkpad (RsnStreamSelector * sel, GstPad * pad)
   return active_sinkpad;
 }
 
-static GList *
-rsn_stream_selector_get_linked_pads (GstPad * pad)
+static void
+rsn_stream_selector_set_active (RsnStreamSelector * sel, GstPad * pad)
 {
-  GstPad *otherpad;
+  GstPad **active_pad_p;
 
-  otherpad = rsn_stream_selector_get_linked_pad (pad, TRUE);
-  if (!otherpad)
-    return NULL;
-  /* need to drop the ref, internal linked pads is not MT safe */
-  gst_object_unref (otherpad);
-  return g_list_append (NULL, otherpad);
+  GST_OBJECT_LOCK (GST_OBJECT_CAST (sel));
+  if (pad != sel->active_sinkpad) {
+    RsnSelectorPad *selpad;
+
+    selpad = GST_SELECTOR_PAD_CAST (pad);
+    /* we can only activate pads that have data received */
+    if (selpad && !selpad->active) {
+      GST_DEBUG_OBJECT (sel, "No data received on pad %" GST_PTR_FORMAT, pad);
+    } else {
+      active_pad_p = &sel->active_sinkpad;
+      gst_object_replace ((GstObject **) active_pad_p, GST_OBJECT_CAST (pad));
+      GST_DEBUG_OBJECT (sel, "New active pad is %" GST_PTR_FORMAT,
+          sel->active_sinkpad);
+    }
+    /* Mark the next buffer as discontinuous */
+    sel->mark_discont = TRUE;
+  }
+  GST_OBJECT_UNLOCK (GST_OBJECT_CAST (sel));
+}
+
+static GstIterator *
+rsn_stream_selector_iterate_linked_pads (GstPad * pad)
+{
+  RsnStreamSelector *sel = RSN_STREAM_SELECTOR (gst_pad_get_parent (pad));
+  GstPad *otherpad = rsn_stream_selector_get_linked_pad (pad, TRUE);
+  GstIterator *it = gst_iterator_new_single (GST_TYPE_PAD, otherpad,
+      (GstCopyFunction) gst_object_ref, (GFreeFunc) gst_object_unref);
+
+  if (otherpad)
+    gst_object_unref (otherpad);
+  gst_object_unref (sel);
+
+  return it;
 }
 
 static GstPad *
@@ -752,8 +756,8 @@ rsn_stream_selector_request_new_pad (GstElement * element,
       GST_DEBUG_FUNCPTR (gst_selector_pad_getcaps));
   gst_pad_set_chain_function (sinkpad,
       GST_DEBUG_FUNCPTR (gst_selector_pad_chain));
-  gst_pad_set_internal_link_function (sinkpad,
-      GST_DEBUG_FUNCPTR (gst_selector_pad_get_linked_pads));
+  gst_pad_set_iterate_internal_links_function (sinkpad,
+      GST_DEBUG_FUNCPTR (rsn_stream_selector_iterate_linked_pads));
   gst_pad_set_bufferalloc_function (sinkpad,
       GST_DEBUG_FUNCPTR (gst_selector_pad_bufferalloc));
 
