@@ -75,6 +75,7 @@ struct _GstSchroEnc
   GstBuffer *seq_header_buffer;
 
   guint64 last_granulepos;
+  guint64 granule_offset;
 };
 
 struct _GstSchroEncClass
@@ -107,7 +108,7 @@ static gboolean gst_schro_enc_start (GstBaseVideoEncoder * base_video_encoder);
 static gboolean gst_schro_enc_stop (GstBaseVideoEncoder * base_video_encoder);
 static gboolean gst_schro_enc_finish (GstBaseVideoEncoder * base_video_encoder,
     GstVideoFrame * frame);
-static gboolean gst_schro_enc_handle_frame (GstBaseVideoEncoder *
+static GstFlowReturn gst_schro_enc_handle_frame (GstBaseVideoEncoder *
     base_video_encoder, GstVideoFrame * frame);
 static GstFlowReturn gst_schro_enc_shape_output (GstBaseVideoEncoder *
     base_video_encoder, GstVideoFrame * frame);
@@ -254,8 +255,46 @@ static gboolean
 gst_schro_enc_set_format (GstBaseVideoEncoder * base_video_encoder,
     GstVideoState * state)
 {
+  GstCaps *caps;
+  GstStructure *structure;
   GstSchroEnc *schro_enc = GST_SCHRO_ENC (base_video_encoder);
 
+  GST_DEBUG ("set_output_caps");
+  caps =
+      gst_pad_get_allowed_caps (GST_BASE_VIDEO_CODEC_SRC_PAD
+      (base_video_encoder));
+  if (caps == NULL) {
+    caps =
+        gst_caps_copy (gst_pad_get_pad_template_caps
+        (GST_BASE_VIDEO_CODEC_SRC_PAD (base_video_encoder)));
+  }
+
+  if (gst_caps_is_empty (caps)) {
+    gst_caps_unref (caps);
+    return FALSE;
+  }
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  if (gst_structure_has_name (structure, "video/x-dirac")) {
+    schro_enc->output_format = GST_SCHRO_ENC_OUTPUT_OGG;
+  } else if (gst_structure_has_name (structure, "video/x-qt-part")) {
+    schro_enc->output_format = GST_SCHRO_ENC_OUTPUT_QUICKTIME;
+  } else if (gst_structure_has_name (structure, "video/x-avi-part")) {
+    schro_enc->output_format = GST_SCHRO_ENC_OUTPUT_AVI;
+  } else if (gst_structure_has_name (structure, "video/x-mp4-part")) {
+    schro_enc->output_format = GST_SCHRO_ENC_OUTPUT_MP4;
+  } else {
+    gst_caps_unref (caps);
+    return FALSE;
+  }
+
+  gst_caps_unref (caps);
+
+  gst_base_video_encoder_set_latency_fields (base_video_encoder,
+      2 * (int) schro_encoder_setting_get_double (schro_enc->encoder,
+          "queue_depth"));
+
   schro_video_format_set_std_video_format (schro_enc->video_format,
       SCHRO_VIDEO_FORMAT_CUSTOM);
 
@@ -375,46 +414,13 @@ gst_schro_enc_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
-/*
- * start is called once the input format is known.  This function
- * must decide on an output format and negotiate it.
- */
 static gboolean
 gst_schro_enc_start (GstBaseVideoEncoder * base_video_encoder)
 {
   GstSchroEnc *schro_enc = GST_SCHRO_ENC (base_video_encoder);
-  GstCaps *caps;
-  GstStructure *structure;
 
-  GST_DEBUG ("set_output_caps");
-  caps =
-      gst_pad_get_allowed_caps (GST_BASE_VIDEO_CODEC_SRC_PAD
-      (base_video_encoder));
+  schro_enc->granule_offset = ~0;
 
-  if (gst_caps_is_empty (caps)) {
-    gst_caps_unref (caps);
-    return FALSE;
-  }
-
-  structure = gst_caps_get_structure (caps, 0);
-
-  if (gst_structure_has_name (structure, "video/x-dirac")) {
-    schro_enc->output_format = GST_SCHRO_ENC_OUTPUT_OGG;
-  } else if (gst_structure_has_name (structure, "video/x-qt-part")) {
-    schro_enc->output_format = GST_SCHRO_ENC_OUTPUT_QUICKTIME;
-  } else if (gst_structure_has_name (structure, "video/x-avi-part")) {
-    schro_enc->output_format = GST_SCHRO_ENC_OUTPUT_AVI;
-  } else if (gst_structure_has_name (structure, "video/x-mp4-part")) {
-    schro_enc->output_format = GST_SCHRO_ENC_OUTPUT_MP4;
-  } else {
-    return FALSE;
-  }
-
-  gst_base_video_encoder_set_latency_fields (base_video_encoder,
-      2 * (int) schro_encoder_setting_get_double (schro_enc->encoder,
-          "queue_depth"));
-
-  gst_caps_unref (caps);
   return TRUE;
 }
 
@@ -447,7 +453,7 @@ gst_schro_enc_finish (GstBaseVideoEncoder * base_video_encoder,
   return TRUE;
 }
 
-static gboolean
+static GstFlowReturn
 gst_schro_enc_handle_frame (GstBaseVideoEncoder * base_video_encoder,
     GstVideoFrame * frame)
 {
@@ -458,7 +464,14 @@ gst_schro_enc_handle_frame (GstBaseVideoEncoder * base_video_encoder,
 
   state = gst_base_video_encoder_get_state (base_video_encoder);
 
-  schro_frame = gst_schro_buffer_wrap (frame->sink_buffer,
+  if (schro_enc->granule_offset == ~0LL) {
+    schro_enc->granule_offset =
+        gst_util_uint64_scale (frame->presentation_timestamp,
+        2 * state->fps_n, GST_SECOND * state->fps_d);
+    GST_DEBUG ("granule offset %" G_GINT64_FORMAT, schro_enc->granule_offset);
+  }
+
+  schro_frame = gst_schro_buffer_wrap (gst_buffer_ref (frame->sink_buffer),
       state->format, state->width, state->height);
 
   GST_DEBUG ("pushing frame %p", frame);
@@ -612,8 +625,8 @@ gst_schro_enc_shape_output_ogg (GstBaseVideoEncoder * base_video_encoder,
 
   dpn = frame->decode_frame_number;
 
-  pt = frame->presentation_frame_number * 2;
-  dt = frame->decode_frame_number * 2;
+  pt = frame->presentation_frame_number * 2 + schro_enc->granule_offset;
+  dt = frame->decode_frame_number * 2 + schro_enc->granule_offset;
   delay = pt - dt;
   dist = frame->distance_from_sync;
 
@@ -621,9 +634,10 @@ gst_schro_enc_shape_output_ogg (GstBaseVideoEncoder * base_video_encoder,
       (int) frame->system_frame_number,
       (int) frame->decode_frame_number, pt, dt, delay, dist);
 
-  granulepos_hi = (((uint64_t) pt - delay) << 9) | ((dist >> 8));
+  granulepos_hi = (((guint64) pt - delay) << 9) | ((dist >> 8));
   granulepos_low = (delay << 9) | (dist & 0xff);
-  GST_DEBUG ("granulepos %lld:%lld", granulepos_hi, granulepos_low);
+  GST_DEBUG ("granulepos %" G_GINT64_FORMAT ":%" G_GINT64_FORMAT, granulepos_hi,
+      granulepos_low);
 
   if (frame->is_eos) {
     GST_BUFFER_OFFSET_END (buf) = schro_enc->last_granulepos;
