@@ -181,8 +181,6 @@ gst_pitch_init (GstPitch * pitch, GstPitchClass * pitch_class)
       GST_DEBUG_FUNCPTR (gst_pad_proxy_getcaps));
   gst_element_add_pad (GST_ELEMENT (pitch), pitch->srcpad);
 
-  gst_element_no_more_pads (GST_ELEMENT (pitch));
-
   pitch->priv->st = new soundtouch::SoundTouch ();
 
   pitch->tempo = 1.0;
@@ -724,7 +722,8 @@ gst_pitch_process_segment (GstPitch * pitch, GstEvent ** event)
     base = 0;
   }
 
-  GST_LOG_OBJECT (pitch->sinkpad, "segment %lld - %lld (%d)", start_value,
+  GST_LOG_OBJECT (pitch->sinkpad,
+      "segment %" G_GINT64_FORMAT " - %" G_GINT64_FORMAT " (%d)", start_value,
       stop_value, format);
 
   if (stream_time_ratio == 0) {
