@@ -201,6 +201,7 @@ struct _GstBaseParsePrivate
   GstFormat duration_fmt;
 
   guint min_frame_size;
+  gboolean passthrough;
 
   gboolean discont;
   gboolean flushing;
@@ -405,6 +406,7 @@ gst_base_parse_init (GstBaseParse * parse, GstBaseParseClass * bclass)
   parse->priv->pad_mode = GST_ACTIVATE_NONE;
   parse->priv->duration = -1;
   parse->priv->min_frame_size = 1;
+  parse->priv->passthrough = FALSE;
   parse->priv->discont = FALSE;
   parse->priv->flushing = FALSE;
   parse->priv->offset = 0;
@@ -917,9 +919,13 @@ gst_base_parse_chain (GstPad * pad, GstBuffer * buffer)
   bclass = GST_BASE_PARSE_GET_CLASS (parse);
 
   if (G_LIKELY (buffer)) {
-    GST_LOG_OBJECT (parse, "buffer size: %d, offset = %lld",
+    GST_LOG_OBJECT (parse, "buffer size: %d, offset = %" G_GINT64_FORMAT,
         GST_BUFFER_SIZE (buffer), GST_BUFFER_OFFSET (buffer));
-    gst_adapter_push (parse->adapter, buffer);
+    if (G_UNLIKELY (parse->priv->passthrough)) {
+      buffer = gst_buffer_make_metadata_writable (buffer);
+      return gst_base_parse_push_buffer (parse, buffer);
+    } else
+      gst_adapter_push (parse->adapter, buffer);
   }
 
   /* Parse and push as many frames as possible */
@@ -1385,7 +1391,7 @@ gst_base_parse_set_duration (GstBaseParse * parse,
   }
   parse->priv->duration = duration;
   parse->priv->duration_fmt = fmt;
-  GST_DEBUG_OBJECT (parse, "set duration: %lld", duration);
+  GST_DEBUG_OBJECT (parse, "set duration: %" G_GINT64_FORMAT, duration);
   GST_BASE_PARSE_UNLOCK (parse);
 }
 
@@ -1410,6 +1416,30 @@ gst_base_parse_set_min_frame_size (GstBaseParse * parse, guint min_size)
   GST_BASE_PARSE_UNLOCK (parse);
 }
 
+/**
+ * gst_base_transform_set_passthrough:
+ * @trans: the #GstBaseTransform to set
+ * @passthrough: boolean indicating passthrough mode.
+ *
+ * Set passthrough mode for this filter by default. This is mostly
+ * useful for filters that do not care about negotiation.
+ *
+ * Always TRUE for filters which don't implement either a transform
+ * or transform_ip method.
+ *
+ * MT safe.
+ */
+void
+gst_base_parse_set_passthrough (GstBaseParse * parse, gboolean passthrough)
+{
+  g_return_if_fail (parse != NULL);
+
+  GST_BASE_PARSE_LOCK (parse);
+  parse->priv->passthrough = passthrough;
+  GST_LOG_OBJECT (parse, "set passthrough: %d", passthrough);
+  GST_BASE_PARSE_UNLOCK (parse);
+}
+
 
 /**
  * gst_base_parse_get_querytypes:
@@ -1632,7 +1662,9 @@ gst_base_parse_handle_seek (GstBaseParse * parse, GstEvent * event)
     return FALSE;
   }
 
-  GST_DEBUG_OBJECT (parse, "seek position %lld in bytes: %lld", cur, seekpos);
+  GST_DEBUG_OBJECT (parse,
+      "seek position %" G_GINT64_FORMAT " in bytes: %" G_GINT64_FORMAT, cur,
+      seekpos);
 
   if (parse->priv->pad_mode == GST_ACTIVATE_PULL) {
     gint64 last_stop;
