@@ -283,6 +283,10 @@ gst_camerabin_set_scene_mode (GstPhotography * photo, GstSceneMode scene_mode)
   if (PHOTOGRAPHY_IS_OK (camera->src_vid_src)) {
     ret = gst_photography_set_scene_mode (GST_PHOTOGRAPHY (camera->src_vid_src),
         scene_mode);
+    if (ret) {
+      gst_photography_get_config (GST_PHOTOGRAPHY (camera->src_vid_src),
+          &camera->photo_settings);
+    }
   }
   return ret;
 }
@@ -389,7 +393,7 @@ gst_camerabin_handle_scene_mode (GstCameraBin * camera, GstSceneMode scene_mode)
       /* Remember frame rate before setting night mode */
       camera->pre_night_fps_n = camera->fps_n;
       camera->pre_night_fps_d = camera->fps_d;
-      g_signal_emit_by_name (camera, "user-res-fps", camera->width,
+      g_signal_emit_by_name (camera, "set-video-resolution-fps", camera->width,
           camera->height, 0, 0, 0);
     } else {
       GST_DEBUG ("night mode already enabled");
@@ -399,7 +403,7 @@ gst_camerabin_handle_scene_mode (GstCameraBin * camera, GstSceneMode scene_mode)
       GST_DEBUG ("disabling night mode, restoring fps to %d/%d",
           camera->pre_night_fps_n, camera->pre_night_fps_d);
       camera->night_mode = FALSE;
-      g_signal_emit_by_name (camera, "user-res-fps", camera->width,
+      g_signal_emit_by_name (camera, "set-video-resolution-fps", camera->width,
           camera->height, camera->pre_night_fps_n, camera->pre_night_fps_d, 0);
     }
   }
