@@ -44,6 +44,20 @@ typedef struct _GstH264ParseClass GstH264ParseClass;
 
 typedef struct _GstNalList GstNalList;
 
+typedef struct _GstH264Sps GstH264Sps;
+typedef struct _GstH264Pps GstH264Pps;
+
+#define MAX_SPS_COUNT	32
+#define MAX_PPS_COUNT   32
+
+#define CLOCK_BASE 9LL
+#define CLOCK_FREQ (CLOCK_BASE * 10000)
+
+#define MPEGTIME_TO_GSTTIME(time) (gst_util_uint64_scale ((time), \
+            GST_MSECOND/10, CLOCK_BASE))
+#define GSTTIME_TO_MPEGTIME(time) (gst_util_uint64_scale ((time), \
+            CLOCK_BASE, GST_MSECOND/10))
+
 struct _GstH264Parse
 {
   GstElement element;
@@ -68,6 +82,40 @@ struct _GstH264Parse
   gboolean have_i_frame;
 
   GstAdapter *adapter;
+
+  /* SPS: sequential parameter set */ 
+  GstH264Sps *sps_buffers[MAX_SPS_COUNT];
+  GstH264Sps *sps; /* Current SPS */ 
+  /* PPS: sequential parameter set */ 
+  GstH264Pps *pps_buffers[MAX_PPS_COUNT];
+  GstH264Pps *pps; /* Current PPS */ 
+
+  /* slice header */ 
+  guint8 first_mb_in_slice;
+  guint8 slice_type;
+  guint8 pps_id;
+  guint32 frame_num;
+  gboolean field_pic_flag;
+  gboolean bottom_field_flag;
+
+  /* SEI: supplemental enhancement messages */ 
+  /* buffering period */ 
+  guint32 initial_cpb_removal_delay[32];
+  /* picture timing */ 
+  guint32 sei_cpb_removal_delay;
+  guint32 sei_dpb_output_delay;
+  guint8 sei_pic_struct;
+  guint8 sei_ct_type; 
+  /* And more... */ 
+
+  /* cached timestamps */ 
+  GstClockTime dts;
+  GstClockTime last_outbuf_dts;
+  GstClockTime ts_trn_nb; /* dts of last buffering period */ 
+  GstClockTime cur_duration; /* duration of the current access unit */ 
+
+  /* for debug purpose */ 
+  guint32 frame_cnt;
 };
 
 struct _GstH264ParseClass
