@@ -39,6 +39,7 @@
   * Fluendo, S.L. All Rights Reserved.
   *
   * Contributor(s): Wim Taymans <wim@fluendo.com>
+  *                 Jan Schmidt <thaytan@noraisin.net>
   */
 
 #ifdef HAVE_CONFIG_H
@@ -52,10 +53,13 @@
 
 #define MAX_DVD_AUDIO_STREAMS       8
 #define MAX_DVD_SUBPICTURE_STREAMS  32
-#define BLOCK_SZ                    4096
+#define BLOCK_SZ                    32768
 #define SCAN_SCR_SZ                 12
 #define SCAN_PTS_SZ                 80
 
+#define SEGMENT_THRESHOLD (300*GST_MSECOND)
+#define VIDEO_SEGMENT_THRESHOLD (500*GST_MSECOND)
+
 typedef enum
 {
   SCAN_SCR,
@@ -73,42 +77,6 @@ typedef enum
 GST_DEBUG_CATEGORY_STATIC (gstflupsdemux_debug);
 #define GST_CAT_DEFAULT (gstflupsdemux_debug)
 
-#ifndef GST_CHECK_VERSION
-#define GST_CHECK_VERSION(major,minor,micro)  \
-    (GST_VERSION_MAJOR > (major) || \
-     (GST_VERSION_MAJOR == (major) && GST_VERSION_MINOR > (minor)) || \
-     (GST_VERSION_MAJOR == (major) && GST_VERSION_MINOR == (minor) && \
-      GST_VERSION_MICRO >= (micro)))
-#endif
-
-#if !GST_CHECK_VERSION(0,10,9)
-#define GST_BUFFER_IS_DISCONT(buffer) \
-    (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT))
-#endif
-
-#if GST_CHECK_VERSION(0,10,6)
-#define HAVE_NEWSEG_FULL
-#else
-static GstBuffer *
-gst_adapter_take_buffer (GstAdapter * adapter, guint nbytes)
-{
-  GstBuffer *buf = NULL;
-
-  if (G_UNLIKELY (nbytes > adapter->size))
-    return NULL;
-
-  buf = gst_buffer_new_and_alloc (nbytes);
-
-  if (G_UNLIKELY (!buf))
-    return NULL;
-
-  /* Slow... */
-  memcpy (GST_BUFFER_DATA (buf), gst_adapter_peek (adapter, nbytes), nbytes);
-
-  return buf;
-}
-#endif
-
 /* elementfactory information */
 static GstElementDetails flups_demux_details = {
   "The Fluendo MPEG Program Stream Demuxer",
@@ -172,11 +140,18 @@ static GstStaticPadTemplate audio_template =
     GST_PAD_SRC,
     GST_PAD_SOMETIMES,
     GST_STATIC_CAPS ("audio/mpeg, "
-        "mpegversion = (int) 1;"
+        "mpegversion = (int) { 1, 4 };"
         "audio/x-private1-lpcm; "
         "audio/x-private1-ac3;" "audio/x-private1-dts;" "audio/ac3")
     );
 
+static GstStaticPadTemplate subpicture_template =
+GST_STATIC_PAD_TEMPLATE ("subpicture_%02x",
+    GST_PAD_SRC,
+    GST_PAD_SOMETIMES,
+    GST_STATIC_CAPS ("video/x-dvd-subpicture")
+    );
+
 static GstStaticPadTemplate private_template =
 GST_STATIC_PAD_TEMPLATE ("private_%d",
     GST_PAD_SRC,
@@ -200,6 +175,7 @@ static void gst_flups_demux_loop (GstPad * pad);
 
 static gboolean gst_flups_demux_src_event (GstPad * pad, GstEvent * event);
 static gboolean gst_flups_demux_src_query (GstPad * pad, GstQuery * query);
+static const GstQueryType *gst_flups_demux_src_query_type (GstPad * pad);
 
 static GstStateChangeReturn gst_flups_demux_change_state (GstElement * element,
     GstStateChange transition);
@@ -209,6 +185,10 @@ static inline gboolean gst_flups_demux_scan_forward_ts (GstFluPSDemux * demux,
 static inline gboolean gst_flups_demux_scan_backward_ts (GstFluPSDemux * demux,
     guint64 * pos, SCAN_MODE mode, guint64 * rts);
 
+static inline void gst_flups_demux_send_segment_updates (GstFluPSDemux * demux,
+    GstClockTime new_time);
+static inline void gst_flups_demux_clear_times (GstFluPSDemux * demux);
+
 static GstElementClass *parent_class = NULL;
 
 /*static guint gst_flups_demux_signals[LAST_SIGNAL] = { 0 };*/
@@ -250,10 +230,14 @@ gst_flups_demux_base_init (GstFluPSDemuxClass * klass)
   klass->sink_template = gst_static_pad_template_get (&sink_template);
   klass->video_template = gst_static_pad_template_get (&video_template);
   klass->audio_template = gst_static_pad_template_get (&audio_template);
+  klass->subpicture_template =
+      gst_static_pad_template_get (&subpicture_template);
   klass->private_template = gst_static_pad_template_get (&private_template);
 
   gst_element_class_add_pad_template (element_class, klass->video_template);
   gst_element_class_add_pad_template (element_class, klass->audio_template);
+  gst_element_class_add_pad_template (element_class,
+      klass->subpicture_template);
   gst_element_class_add_pad_template (element_class, klass->private_template);
   gst_element_class_add_pad_template (element_class, klass->sink_template);
 
@@ -297,6 +281,9 @@ gst_flups_demux_init (GstFluPSDemux * demux)
 
   demux->streams =
       g_malloc0 (sizeof (GstFluPSStream *) * (GST_FLUPS_DEMUX_MAX_STREAMS));
+  demux->streams_found =
+      g_malloc0 (sizeof (GstFluPSStream *) * (GST_FLUPS_DEMUX_MAX_STREAMS));
+  demux->found_count = 0;
 
 }
 
@@ -305,6 +292,7 @@ gst_flups_demux_finalize (GstFluPSDemux * demux)
 {
   gst_flups_demux_reset (demux);
   g_free (demux->streams);
+  g_free (demux->streams_found);
 
   G_OBJECT_CLASS (parent_class)->finalize (G_OBJECT (demux));
 }
@@ -327,6 +315,9 @@ gst_flups_demux_reset (GstFluPSDemux * demux)
       demux->streams[i] = NULL;
     }
   }
+  memset (demux->streams_found, 0,
+      sizeof (GstFluPSStream *) * (GST_FLUPS_DEMUX_MAX_STREAMS));
+  demux->found_count = 0;
   p_ev = &demux->lang_codes;
 
   gst_event_replace (p_ev, NULL);
@@ -340,6 +331,7 @@ gst_flups_demux_create_stream (GstFluPSDemux * demux, gint id, gint stream_type)
   gchar *name;
   GstFluPSDemuxClass *klass = GST_FLUPS_DEMUX_GET_CLASS (demux);
   GstCaps *caps;
+  GstClockTime threshold = SEGMENT_THRESHOLD;
 
   name = NULL;
   template = NULL;
@@ -368,6 +360,7 @@ gst_flups_demux_create_stream (GstFluPSDemux * demux, gint id, gint stream_type)
       caps = gst_caps_new_simple ("video/mpeg",
           "mpegversion", G_TYPE_INT, mpeg_version,
           "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
+      threshold = VIDEO_SEGMENT_THRESHOLD;
       break;
     }
     case ST_AUDIO_MPEG1:
@@ -381,12 +374,18 @@ gst_flups_demux_create_stream (GstFluPSDemux * demux, gint id, gint stream_type)
     case ST_PRIVATE_DATA:
     case ST_MHEG:
     case ST_DSMCC:
+      break;
     case ST_AUDIO_AAC:
+      template = klass->audio_template;
+      name = g_strdup_printf ("audio_%02x", id);
+      caps = gst_caps_new_simple ("audio/mpeg",
+          "mpegversion", G_TYPE_INT, 4, NULL);
       break;
     case ST_VIDEO_H264:
       template = klass->video_template;
       name = g_strdup_printf ("video_%02x", id);
       caps = gst_caps_new_simple ("video/x-h264", NULL);
+      threshold = VIDEO_SEGMENT_THRESHOLD;
       break;
     case ST_PS_AUDIO_AC3:
       template = klass->audio_template;
@@ -404,6 +403,9 @@ gst_flups_demux_create_stream (GstFluPSDemux * demux, gint id, gint stream_type)
       caps = gst_caps_new_simple ("audio/x-private1-lpcm", NULL);
       break;
     case ST_PS_DVD_SUBPICTURE:
+      template = klass->subpicture_template;
+      name = g_strdup_printf ("subpicture_%02x", id);
+      caps = gst_caps_new_simple ("video/x-dvd-subpicture", NULL);
       break;
     case ST_GST_AUDIO_RAWA52:
       template = klass->audio_template;
@@ -424,10 +426,13 @@ gst_flups_demux_create_stream (GstFluPSDemux * demux, gint id, gint stream_type)
   stream->notlinked = FALSE;
   stream->type = stream_type;
   stream->pad = gst_pad_new_from_template (template, name);
+  stream->segment_thresh = threshold;
   gst_pad_set_event_function (stream->pad,
       GST_DEBUG_FUNCPTR (gst_flups_demux_src_event));
   gst_pad_set_query_function (stream->pad,
       GST_DEBUG_FUNCPTR (gst_flups_demux_src_query));
+  gst_pad_set_query_type_function (stream->pad,
+      GST_DEBUG_FUNCPTR (gst_flups_demux_src_query_type));
   gst_pad_use_fixed_caps (stream->pad);
   gst_pad_set_caps (stream->pad, caps);
   gst_caps_unref (caps);
@@ -454,6 +459,7 @@ gst_flups_demux_get_stream (GstFluPSDemux * demux, gint id, gint type)
     gst_element_add_pad (GST_ELEMENT (demux), stream->pad);
 
     demux->streams[id] = stream;
+    demux->streams_found[demux->found_count++] = stream;
   }
   return stream;
 
@@ -477,13 +483,13 @@ gst_flups_demux_send_data (GstFluPSDemux * demux, GstFluPSStream * stream,
     goto no_stream;
 
   /* timestamps */
-  if (demux->next_pts != G_MAXUINT64)
+  if (G_UNLIKELY (demux->next_pts != G_MAXUINT64))
     timestamp = MPEGTIME_TO_GSTTIME (demux->next_pts);
   else
     timestamp = GST_CLOCK_TIME_NONE;
 
   /* discont */
-  if (stream->need_segment) {
+  if (G_UNLIKELY (stream->need_segment)) {
     gint64 time, start, stop;
     GstEvent *newsegment;
 
@@ -519,7 +525,6 @@ gst_flups_demux_send_data (GstFluPSDemux * demux, GstFluPSStream * stream,
     else
       time = 0;
 
-#ifdef HAVE_NEWSEG_FULL
     GST_INFO_OBJECT (demux, "sending new segment: rate %g applied_rate %g "
         "start: %" GST_TIME_FORMAT ", stop: %" GST_TIME_FORMAT
         ", time: %" GST_TIME_FORMAT " to pad %" GST_PTR_FORMAT,
@@ -530,16 +535,6 @@ gst_flups_demux_send_data (GstFluPSDemux * demux, GstFluPSStream * stream,
     newsegment = gst_event_new_new_segment_full (FALSE,
         demux->sink_segment.rate, demux->sink_segment.applied_rate,
         GST_FORMAT_TIME, start, stop, time);
-#else
-    GST_INFO_OBJECT (demux, "sending new segment: rate %g "
-        "start: %" GST_TIME_FORMAT ", stop: %" GST_TIME_FORMAT
-        ", time: %" GST_TIME_FORMAT " to pad %" GST_PTR_FORMAT,
-        demux->sink_segment.rate, GST_TIME_ARGS (start),
-        GST_TIME_ARGS (stop), GST_TIME_ARGS (time), stream->pad);
-
-    newsegment = gst_event_new_new_segment (FALSE,
-        demux->sink_segment.rate, GST_FORMAT_TIME, start, stop, time);
-#endif
 
     gst_pad_push_event (stream->pad, newsegment);
 
@@ -559,6 +554,20 @@ gst_flups_demux_send_data (GstFluPSDemux * demux, GstFluPSStream * stream,
       GST_TIME_ARGS (demux->src_segment.last_stop),
       GST_TIME_ARGS (MPEGTIME_TO_GSTTIME (demux->current_scr)));
 
+  if (demux->src_segment.last_stop != GST_CLOCK_TIME_NONE) {
+    GstClockTime new_time = demux->base_time + demux->src_segment.last_stop;
+
+    if (stream->last_ts == GST_CLOCK_TIME_NONE || stream->last_ts < new_time) {
+#if 0
+      g_print ("last_ts update on pad %s to time %" GST_TIME_FORMAT "\n",
+          GST_PAD_NAME (stream->pad), GST_TIME_ARGS (cur_scr_time));
+#endif
+      stream->last_ts = new_time;
+    }
+
+    gst_flups_demux_send_segment_updates (demux, new_time);
+  }
+
   /* Set the buffer discont flag, and clear discont state on the stream */
   if (stream->discont) {
     GST_DEBUG_OBJECT (demux, "marking discont buffer");
@@ -588,17 +597,17 @@ no_stream:
   }
 }
 
-static void
+static inline void
 gst_flups_demux_mark_discont (GstFluPSDemux * demux, gboolean discont,
     gboolean need_segment)
 {
-  gint id;
+  gint i, count = demux->found_count;
 
   /* mark discont on all streams */
-  for (id = 0; id < GST_FLUPS_DEMUX_MAX_STREAMS; id++) {
-    GstFluPSStream *stream = demux->streams[id];
+  for (i = 0; i < count; i++) {
+    GstFluPSStream *stream = demux->streams_found[i];
 
-    if (stream) {
+    if (G_LIKELY (stream)) {
       stream->discont |= discont;
       stream->need_segment |= need_segment;
       GST_DEBUG_OBJECT (demux, "marked stream as discont %d, need_segment %d",
@@ -607,14 +616,14 @@ gst_flups_demux_mark_discont (GstFluPSDemux * demux, gboolean discont,
   }
 }
 
-static gboolean
+static inline gboolean
 gst_flups_demux_send_event (GstFluPSDemux * demux, GstEvent * event)
 {
-  gint id;
+  gint i, count = demux->found_count;
   gboolean ret = FALSE;
 
-  for (id = 0; id < GST_FLUPS_DEMUX_MAX_STREAMS; id++) {
-    GstFluPSStream *stream = demux->streams[id];
+  for (i = 0; i < count; i++) {
+    GstFluPSStream *stream = demux->streams_found[i];
 
     if (stream && !stream->notlinked) {
       (void) gst_event_ref (event);
@@ -675,7 +684,7 @@ gst_flups_demux_handle_dvd_event (GstFluPSDemux * demux, GstEvent * event)
           break;
         case 0x2:
         case 0x3:
-          /* MPEG audio without and with extension stream are 
+          /* MPEG audio without and with extension stream are
            * treated the same */
           stream_id = 0xC0 + i;
           temp = gst_flups_demux_get_stream (demux, stream_id, ST_AUDIO_MPEG1);
@@ -731,30 +740,108 @@ gst_flups_demux_flush (GstFluPSDemux * demux)
   gst_adapter_clear (demux->adapter);
   gst_adapter_clear (demux->rev_adapter);
   gst_pes_filter_drain (&demux->filter);
+  gst_flups_demux_clear_times (demux);
   demux->adapter_offset = G_MAXUINT64;
   demux->current_scr = G_MAXUINT64;
   demux->bytes_since_scr = 0;
 }
 
-static void
-gst_flups_demux_close_segment (GstFluPSDemux * demux)
+static inline void
+gst_flups_demux_clear_times (GstFluPSDemux * demux)
 {
-  gint id;
+  gint i, count = demux->found_count;
+
+  /* Clear the last ts for all streams */
+  for (i = 0; i < count; i++) {
+    GstFluPSStream *stream = demux->streams_found[i];
+
+    if (G_LIKELY (stream)) {
+      stream->last_seg_start = stream->last_ts = GST_CLOCK_TIME_NONE;
+    }
+  }
+}
+
+static inline void
+gst_flups_demux_send_segment_updates (GstFluPSDemux * demux,
+    GstClockTime new_start)
+{
+  GstClockTime base_time, stop, time;
+  gint i, count = demux->found_count;
   GstEvent *event = NULL;
 
-#if POST_10_10
+  /* Advance all lagging streams by sending a segment update */
+  if ((base_time = demux->base_time) == GST_CLOCK_TIME_NONE)
+    base_time = 0;
+
+  stop = demux->src_segment.stop;
+  if (stop != GST_CLOCK_TIME_NONE)
+    stop += base_time;
+
+  if (new_start > stop)
+    return;
+
+  time = demux->src_segment.time;
+  time += new_start - (demux->src_segment.start + base_time);
+
+  /* FIXME: Handle reverse playback */
+  for (i = 0; i < count; i++) {
+    GstFluPSStream *stream = demux->streams_found[i];
+
+    if (stream) {
+      if (stream->last_ts == GST_CLOCK_TIME_NONE ||
+          stream->last_ts < demux->src_segment.start + base_time)
+        stream->last_ts = demux->src_segment.start + base_time;
+
+      if (stream->last_ts + stream->segment_thresh < new_start) {
+#if 0
+        g_print ("Segment update to pad %s time %" GST_TIME_FORMAT " stop now %"
+            GST_TIME_FORMAT "\n", GST_PAD_NAME (stream->pad),
+            GST_TIME_ARGS (new_start), GST_TIME_ARGS (demux->src_segment.stop));
+#endif
+        GST_DEBUG_OBJECT (demux,
+            "Segment update to pad %s time %" GST_TIME_FORMAT,
+            GST_PAD_NAME (stream->pad), GST_TIME_ARGS (new_start));
+        if (event == NULL) {
+          event = gst_event_new_new_segment_full (TRUE,
+              demux->src_segment.rate, demux->src_segment.applied_rate,
+              GST_FORMAT_TIME, new_start, stop, time);
+        }
+        gst_event_ref (event);
+        gst_pad_push_event (stream->pad, event);
+        stream->last_seg_start = stream->last_ts = new_start;
+        stream->need_segment = FALSE;
+      }
+    }
+  }
+
+  if (event)
+    gst_event_unref (event);
+}
+
+static inline void
+gst_flups_demux_close_segment (GstFluPSDemux * demux)
+{
+  gint i, count = demux->found_count;
+  GstEvent *event = NULL;
+  guint64 base_time;
+
   GST_INFO_OBJECT (demux, "closing running segment %" GST_SEGMENT_FORMAT,
       &demux->src_segment);
-#endif
+
+  /* FIXME: Need to send a different segment-close to each pad where the
+   * last_seg_start != clock_time_none, as that indicates a sparse-stream
+   * event was sent there */
+
+  if ((base_time = demux->base_time) == (guint64) - 1)
+    base_time = 0;
 
   /* Close the current segment for a linear playback */
   if (demux->src_segment.rate >= 0) {
     /* for forward playback, we played from start to last_stop */
     event = gst_event_new_new_segment (TRUE,
         demux->src_segment.rate, demux->src_segment.format,
-        demux->src_segment.start + demux->base_time,
-        demux->src_segment.last_stop + demux->base_time,
-        demux->src_segment.time);
+        demux->src_segment.start + base_time,
+        demux->src_segment.last_stop + base_time, demux->src_segment.time);
   } else {
     gint64 stop;
 
@@ -764,13 +851,13 @@ gst_flups_demux_close_segment (GstFluPSDemux * demux)
     /* for reverse playback, we played from stop to last_stop. */
     event = gst_event_new_new_segment (TRUE,
         demux->src_segment.rate, demux->src_segment.format,
-        demux->src_segment.last_stop + demux->base_time,
-        stop + demux->base_time, demux->src_segment.last_stop);
+        demux->src_segment.last_stop + base_time,
+        stop + base_time, demux->src_segment.last_stop);
   }
 
   if (event) {
-    for (id = 0; id < GST_FLUPS_DEMUX_MAX_STREAMS; id++) {
-      GstFluPSStream *stream = demux->streams[id];
+    for (i = 0; i < count; i++) {
+      GstFluPSStream *stream = demux->streams_found[i];
 
       if (stream && !stream->notlinked && !stream->need_segment) {
         (void) gst_event_ref (event);
@@ -790,6 +877,12 @@ gst_flups_demux_close_segment (GstFluPSDemux * demux)
   }
 }
 
+static inline gboolean
+have_open_streams (GstFluPSDemux * demux)
+{
+  return (demux->streams_found[0] != NULL);
+}
+
 static gboolean
 gst_flups_demux_sink_event (GstPad * pad, GstEvent * event)
 {
@@ -817,7 +910,6 @@ gst_flups_demux_sink_event (GstPad * pad, GstEvent * event)
       /* Close current segment */
       gst_flups_demux_close_segment (demux);
 
-#ifdef HAVE_NEWSEG_FULL
       {
         gdouble arate;
 
@@ -835,19 +927,6 @@ gst_flups_demux_sink_event (GstPad * pad, GstEvent * event)
         }
 
       }
-#else
-      gst_event_parse_new_segment (event, &update, &rate, &format,
-          &start, &stop, &time);
-      gst_segment_set_newsegment (&demux->sink_segment, update, rate,
-          format, start, stop, time);
-      if (format == GST_FORMAT_BYTES && demux->scr_rate_n != G_MAXUINT64
-          && demux->scr_rate_d != G_MAXUINT64) {
-
-        gst_segment_set_newsegment (&demux->src_segment, update, rate,
-            GST_FORMAT_TIME, BYTES_TO_GSTTIME (start), BYTES_TO_GSTTIME (stop),
-            BYTES_TO_GSTTIME (time));
-      }
-#endif
 
       GST_INFO_OBJECT (demux, "received new segment: rate %g "
           "format %d, start: %" G_GINT64_FORMAT ", stop: %" G_GINT64_FORMAT
@@ -861,10 +940,11 @@ gst_flups_demux_sink_event (GstPad * pad, GstEvent * event)
     }
     case GST_EVENT_EOS:
       GST_INFO_OBJECT (demux, "Received EOS");
-      if (!gst_flups_demux_send_event (demux, event)) {
-        GST_WARNING_OBJECT (demux, "failed pushing EOS on streams");
+      if (!gst_flups_demux_send_event (demux, event)
+          && !have_open_streams (demux)) {
+        GST_WARNING_OBJECT (demux, "EOS and no streams open");
         GST_ELEMENT_ERROR (demux, STREAM, FAILED,
-            ("Internal data stream error."), ("Can't push EOS downstream"));
+            ("Internal data stream error."), ("No valid streams detected"));
       }
       break;
     case GST_EVENT_CUSTOM_DOWNSTREAM:
@@ -966,14 +1046,18 @@ gst_flups_demux_do_seek (GstFluPSDemux * demux, GstSegment * seeksegment)
   guint64 scr_rate_n = demux->last_scr_offset - demux->first_scr_offset;
   guint64 scr_rate_d = demux->last_scr - demux->first_scr;
 
+  /* In some clips the PTS values are completely unaligned with SCR values.
+   * To improve the seek in that situation we apply a factor considering the
+   * relationship between last PTS and last SCR */
+  if (demux->last_scr > demux->last_pts)
+    scr = gst_util_uint64_scale (scr, demux->last_scr, demux->last_pts);
+
   scr = MIN (demux->last_scr, scr);
   scr = MAX (demux->first_scr, scr);
   fscr = scr;
 
-#if POST_10_10
   GST_INFO_OBJECT (demux, "sink segment configured %" GST_SEGMENT_FORMAT
       ", trying to go at SCR: %" G_GUINT64_FORMAT, &demux->sink_segment, scr);
-#endif
 
   offset = MIN (gst_util_uint64_scale (scr, scr_rate_n, scr_rate_d),
       demux->sink_segment.stop);
@@ -1030,6 +1114,7 @@ gst_flups_demux_handle_seek_pull (GstFluPSDemux * demux, GstEvent * event)
   if (flush) {
     /* Flush start up and downstream to make sure data flow and loops are
        idle */
+    demux->flushing = TRUE;
     gst_flups_demux_send_event (demux, gst_event_new_flush_start ());
     gst_pad_push_event (demux->sinkpad, gst_event_new_flush_start ());
   } else {
@@ -1042,25 +1127,22 @@ gst_flups_demux_handle_seek_pull (GstFluPSDemux * demux, GstEvent * event)
 
   if (flush) {
     /* Stop flushing upstream we need to pull */
+    demux->flushing = FALSE;
     gst_pad_push_event (demux->sinkpad, gst_event_new_flush_stop ());
   }
 
   /* Work on a copy until we are sure the seek succeeded. */
   memcpy (&seeksegment, &demux->src_segment, sizeof (GstSegment));
 
-#if POST_10_10
   GST_DEBUG_OBJECT (demux, "segment before configure %" GST_SEGMENT_FORMAT,
       &demux->src_segment);
-#endif
 
   /* Apply the seek to our segment */
   gst_segment_set_seek (&seeksegment, rate, format, flags,
       start_type, start, stop_type, stop, &update);
 
-#if POST_10_10
   GST_DEBUG_OBJECT (demux, "seek segment configured %" GST_SEGMENT_FORMAT,
       &seeksegment);
-#endif
 
   if (flush || seeksegment.last_stop != demux->src_segment.last_stop) {
     /* Do the actual seeking */
@@ -1078,10 +1160,8 @@ gst_flups_demux_handle_seek_pull (GstFluPSDemux * demux, GstEvent * event)
   /* update the rate in our src segment */
   demux->sink_segment.rate = rate;
 
-#if POST_10_10
   GST_DEBUG_OBJECT (demux, "seek segment adjusted %" GST_SEGMENT_FORMAT,
       &seeksegment);
-#endif
 
   if (flush) {
     /* Stop flushing, the sinks are at time 0 now */
@@ -1157,6 +1237,19 @@ gst_flups_demux_src_event (GstPad * pad, GstEvent * event)
   return res;
 }
 
+static const GstQueryType *
+gst_flups_demux_src_query_type (GstPad * pad)
+{
+  static const GstQueryType types[] = {
+    GST_QUERY_POSITION,
+    GST_QUERY_DURATION,
+    GST_QUERY_SEEKING,
+    0
+  };
+
+  return types;
+}
+
 static gboolean
 gst_flups_demux_src_query (GstPad * pad, GstQuery * query)
 {
@@ -1223,7 +1316,7 @@ gst_flups_demux_src_query (GstPad * pad, GstQuery * query)
         break;
       }
 
-      /* Upstream didn't know, so we can only answer TIME queries from 
+      /* Upstream didn't know, so we can only answer TIME queries from
        * here on */
       if (format != GST_FORMAT_TIME) {
         GST_DEBUG_OBJECT (demux, "duration not supported for format %d",
@@ -1261,11 +1354,66 @@ gst_flups_demux_src_query (GstPad * pad, GstQuery * query)
       res = TRUE;
       break;
     }
+    case GST_QUERY_SEEKING:{
+      GstFormat fmt;
+
+      gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
+
+      res = TRUE;
+      if (demux->random_access) {
+        /* In pull mode we can seek in TIME format if we have the SCR */
+        if (fmt != GST_FORMAT_TIME || demux->scr_rate_n == G_MAXUINT64
+            || demux->scr_rate_d == G_MAXUINT64)
+          gst_query_set_seeking (query, fmt, FALSE, -1, -1);
+        else
+          gst_query_set_seeking (query, fmt, TRUE, 0, -1);
+      } else {
+        if (fmt == GST_FORMAT_BYTES) {
+          /* Seeking in BYTES format not supported at all */
+          gst_query_set_seeking (query, fmt, FALSE, -1, -1);
+        } else {
+          GstQuery *peerquery;
+          gboolean seekable;
+
+          /* Then ask upstream */
+          res = gst_pad_peer_query (demux->sinkpad, query);
+          if (res) {
+            /* If upstream can handle seeks we're done, if it
+             * can't we still have our TIME->BYTES conversion seek
+             */
+            gst_query_parse_seeking (query, NULL, &seekable, NULL, NULL);
+            if (seekable || fmt != GST_FORMAT_TIME)
+              goto beach;
+          }
+
+          /* We can seek if upstream supports BYTES seeks and we
+           * have the SCR
+           */
+          peerquery = gst_query_new_seeking (GST_FORMAT_BYTES);
+          res = gst_pad_peer_query (demux->sinkpad, peerquery);
+          if (!res || demux->scr_rate_n == G_MAXUINT64
+              || demux->scr_rate_d == G_MAXUINT64) {
+            gst_query_set_seeking (query, fmt, FALSE, -1, -1);
+          } else {
+            gst_query_parse_seeking (peerquery, NULL, &seekable, NULL, NULL);
+            if (seekable)
+              gst_query_set_seeking (query, GST_FORMAT_TIME, TRUE, 0, -1);
+            else
+              gst_query_set_seeking (query, fmt, FALSE, -1, -1);
+          }
+
+          gst_query_unref (peerquery);
+          res = TRUE;
+        }
+      }
+      break;
+    }
     default:
       res = gst_pad_query_default (pad, query);
       break;
   }
 
+beach:
   gst_object_unref (demux);
 
   return res;
@@ -1292,7 +1440,7 @@ gst_flups_demux_reset_psm (GstFluPSDemux * demux)
   FILL_TYPE (0x40, 0x7f, -1);
   FILL_TYPE (0x80, 0x87, ST_PS_AUDIO_AC3);
   FILL_TYPE (0x88, 0x9f, ST_PS_AUDIO_DTS);
-  FILL_TYPE (0xa0, 0xbf, ST_PS_AUDIO_LPCM);
+  FILL_TYPE (0xa0, 0xaf, ST_PS_AUDIO_LPCM);
   FILL_TYPE (0xbd, 0xbd, -1);
   FILL_TYPE (0xc0, 0xdf, ST_AUDIO_MPEG1);
   FILL_TYPE (0xe0, 0xef, ST_GST_VIDEO_MPEG1_OR_2);
@@ -1301,6 +1449,33 @@ gst_flups_demux_reset_psm (GstFluPSDemux * demux)
 #undef FILL_TYPE
 }
 
+/* ISO/IEC 13818-1:
+ * pack_header() {
+ *     pack_start_code                                   32  bslbf  -+
+ *     '01'                                               2  bslbf   |
+ *     system_clock_reference_base [32..30]               3  bslbf   |
+ *     marker_bit                                         1  bslbf   |
+ *     system_clock_reference_base [29..15]              15  bslbf   |
+ *     marker_bit                                         1  bslbf   |
+ *     system_clock_reference_base [14..0]               15  bslbf   |
+ *     marker_bit                                         1  bslbf   | 112 bits
+ *     system_clock_reference_extension                   9  ubslbf  |
+ *     marker_bit                                         1  bslbf   |
+ *     program_mux_rate                                  22  ubslbf  |
+ *     marker_bit                                         1  bslbf   |
+ *     marker_bit                                         1  bslbf   |
+ *     reserved                                           5  bslbf   |
+ *     pack_stuffing_length                               3  ubslbf -+
+ *
+ *     for (i = 0; i < pack_stuffing_length; i++) {
+ *         stuffing_byte '1111 1111'                      8  bslbf
+ *     }
+ *
+ * 112 bits = 14 bytes, as max value for pack_stuffing_length is 7, then
+ * in total it's needed 14 + 7 = 21 bytes.
+ */
+#define PACK_START_SIZE     21
+
 static GstFlowReturn
 gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
 {
@@ -1310,21 +1485,24 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
   guint64 scr, scr_adjusted, new_rate;
   guint64 scr_rate_n;
   guint64 scr_rate_d;
+  guint avail = gst_adapter_available (demux->adapter);
 
   GST_DEBUG ("parsing pack start");
 
-  /* fixed length to begin with, start code and two scr values */
-  length = 8 + 4;
-
-  if (!(data = gst_adapter_peek (demux->adapter, length)))
+  if (G_UNLIKELY (avail < PACK_START_SIZE))
     goto need_more_data;
 
+  data = gst_adapter_peek (demux->adapter, PACK_START_SIZE);
+
   /* skip start code */
   data += 4;
 
   scr1 = GUINT32_FROM_BE (*(guint32 *) data);
   scr2 = GUINT32_FROM_BE (*(guint32 *) (data + 4));
 
+  /* fixed length to begin with, start code and two scr values */
+  length = 8 + 4;
+
   /* start parsing the stream */
   if ((*data & 0xc0) == 0x40) {
     guint32 scr_ext;
@@ -1336,13 +1514,11 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
 
     /* mpeg2 has more data */
     length += 2;
-    if (gst_adapter_available (demux->adapter) < length)
-      goto need_more_data;
 
     /* :2=01 ! scr:3 ! marker:1==1 ! scr:15 ! marker:1==1 ! scr:15 */
 
     /* check markers */
-    if ((scr1 & 0xc4000400) != 0x44000400)
+    if (G_UNLIKELY ((scr1 & 0xc4000400) != 0x44000400))
       goto lost_sync;
 
     scr = ((guint64) scr1 & 0x38000000) << 3;
@@ -1351,14 +1527,15 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
     scr |= ((guint64) scr2 & 0xf8000000) >> 27;
 
     /* marker:1==1 ! scr_ext:9 ! marker:1==1 */
-    if ((scr2 & 0x04010000) != 0x04010000)
+    if (G_UNLIKELY ((scr2 & 0x04010000) != 0x04010000))
       goto lost_sync;
 
     scr_ext = (scr2 & 0x03fe0000) >> 17;
     /* We keep the offset of this scr */
     demux->cur_scr_offset = demux->adapter_offset + 12;
 
-    GST_DEBUG_OBJECT (demux, "SCR: 0x%08llx SCRE: 0x%08x", scr, scr_ext);
+    GST_DEBUG_OBJECT (demux, "SCR: 0x%08" G_GINT64_MODIFIER "x SCRE: 0x%08x",
+        scr, scr_ext);
 
     if (scr_ext) {
       scr = (scr * 300 + scr_ext % 300) / 300;
@@ -1368,7 +1545,7 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
     data += 6;
     /* PMR:22 ! :2==11 ! reserved:5 ! stuffing_len:3 */
     next32 = (GUINT32_FROM_BE ((*(guint32 *) data)));
-    if ((next32 & 0x00000300) != 0x00000300)
+    if (G_UNLIKELY ((next32 & 0x00000300) != 0x00000300))
       goto lost_sync;
 
     new_rate = (next32 & 0xfffffc00) >> 10;
@@ -1377,6 +1554,7 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
     GST_DEBUG_OBJECT (demux, "stuffing bytes: %d", stuffing_bytes);
 
     data += 4;
+    length += stuffing_bytes;
     while (stuffing_bytes--) {
       if (*data++ != 0xff)
         goto lost_sync;
@@ -1386,10 +1564,10 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
     demux->is_mpeg2_pack = FALSE;
 
     /* check markers */
-    if ((scr1 & 0xf1000100) != 0x21000100)
+    if (G_UNLIKELY ((scr1 & 0xf1000100) != 0x21000100))
       goto lost_sync;
 
-    if ((scr2 & 0x01800001) != 0x01800001)
+    if (G_UNLIKELY ((scr2 & 0x01800001) != 0x01800001))
       goto lost_sync;
 
     /* :4=0010 ! scr:3 ! marker:1==1 ! scr:15 ! marker:1==1 ! scr:15 ! marker:1==1 */
@@ -1418,14 +1596,14 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
       GST_TIME_ARGS (MPEGTIME_TO_GSTTIME ((guint64) scr)));
 
   /* keep the first src in order to calculate delta time */
-  if (demux->first_scr == G_MAXUINT64) {
+  if (G_UNLIKELY (demux->first_scr == G_MAXUINT64)) {
     demux->first_scr = scr;
     demux->first_scr_offset = demux->cur_scr_offset;
     demux->base_time = MPEGTIME_TO_GSTTIME (demux->first_scr);
     /* at begin consider the new_rate as the scr rate, bytes/clock ticks */
     scr_rate_n = new_rate;
     scr_rate_d = CLOCK_FREQ;
-  } else if (demux->first_scr_offset != demux->cur_scr_offset) {
+  } else if (G_LIKELY (demux->first_scr_offset != demux->cur_scr_offset)) {
     /* estimate byte rate related to the SCR */
     scr_rate_n = demux->cur_scr_offset - demux->first_scr_offset;
     scr_rate_d = scr_adjusted - demux->first_scr;
@@ -1444,7 +1622,7 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
       scr_rate_n, scr_rate_d, (float) scr_rate_n / scr_rate_d);
 
   /* adjustment of the SCR */
-  if (demux->current_scr != G_MAXUINT64) {
+  if (G_LIKELY (demux->current_scr != G_MAXUINT64)) {
     gint64 diff;
     guint64 old_scr, old_mux_rate, bss, adjust = 0;
 
@@ -1455,8 +1633,7 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
     /* Bytes since SCR is the amount we placed in the adapter since then
      * (demux->bytes_since_scr) minus the amount remaining in the adapter,
      * clamped to >= 0 */
-    bss = MAX (0, (gint) (demux->bytes_since_scr -
-            gst_adapter_available (demux->adapter)));
+    bss = MAX (0, (gint) (demux->bytes_since_scr - avail));
 
     /* estimate the new SCR using the previous one according the notes
        on point 2.5.2.2 of the ISO/IEC 13818-1 document */
@@ -1475,14 +1652,14 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
 
     /* calculate the absolute deference between the last scr and
        the new one */
-    if (old_scr > scr_adjusted)
+    if (G_UNLIKELY (old_scr > scr_adjusted))
       diff = old_scr - scr_adjusted;
     else
       diff = scr_adjusted - old_scr;
 
     /* if the difference is more than 1 second we need to reconfigure
        adjustment */
-    if (diff > CLOCK_FREQ) {
+    if (G_UNLIKELY (diff > CLOCK_FREQ)) {
       demux->scr_adjust = demux->next_scr - scr;
       GST_DEBUG_OBJECT (demux, "discont found, diff: %" G_GINT64_FORMAT
           ", adjust %" G_GINT64_FORMAT, diff, demux->scr_adjust);
@@ -1503,7 +1680,7 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
 
   /* Reset the bytes_since_scr value to count the data remaining in the
    * adapter */
-  demux->bytes_since_scr = gst_adapter_available (demux->adapter);
+  demux->bytes_since_scr = avail;
 
   gst_adapter_flush (demux->adapter, length);
   ADAPTER_OFFSET_FLUSH (length);
@@ -1521,6 +1698,32 @@ need_more_data:
   }
 }
 
+/* ISO/IEC 13818-1:
+ * system_header () {
+ *     system_header_start_code                          32  bslbf  -+
+ *     header_length                                     16  uimsbf  |
+ *     marker_bit                                         1  bslbf   |
+ *     rate_bound                                        22  uimsbf  |
+ *     marker_bit                                         1  bslbf   |
+ *     audio_bound                                        6  uimsbf  |
+ *     fixed_flag                                         1  bslbf   |
+ *     CSPS_flag                                          1  bslbf   | 96 bits
+ *     system_audio_lock_flag                             1  bslbf   |
+ *     system_video_lock_flag                             1  bslbf   |
+ *     marker_bit                                         1  bslbf   |
+ *     video_bound                                        5  uimsbf  |
+ *     packet_rate_restriction_flag                       1  bslbf   |
+ *     reserved_bits                                      7  bslbf  -+
+ *     while (nextbits () = = '1') {
+ *         stream_id                                      8  uimsbf -+
+ *         '11'                                           2  bslbf   | 24 bits
+ *         P-STD_buffer_bound_scale                       1  bslbf   |
+ *         P-STD_buffer_size_bound                       13  uimsbf -+
+ *     }
+ * }
+ * 96 bits = 12 bytes, 24 bits = 3 bytes.
+ */
+
 static GstFlowReturn
 gst_flups_demux_parse_sys_head (GstFluPSDemux * demux)
 {
@@ -1760,7 +1963,15 @@ gst_flups_demux_parse_psm (GstFluPSDemux * demux)
 
     GST_DEBUG_OBJECT (demux, "Stream type %02X with id %02X and %u bytes info",
         stream_type, stream_id, stream_info_length);
-    demux->psm[stream_id] = stream_type;
+    if (G_LIKELY (stream_id != 0xbd))
+      demux->psm[stream_id] = stream_type;
+    else {
+      /* Ignore stream type for private_stream_1 and discover it looking at
+       * the stream data.
+       * Fixes demuxing some clips with lpcm that was wrongly declared as
+       * mpeg audio */
+      GST_DEBUG_OBJECT (demux, "stream type for private_stream_1 ignored");
+    }
     es_map_base += stream_info_length;
   }
 
@@ -1812,7 +2023,7 @@ gst_flups_demux_data_cb (GstPESFilter * filter, gboolean first,
       if (start_code == ID_PRIVATE_STREAM_1 && datalen >= 2) {
         guint8 nframes;
 
-        /* VDR writes A52 streams without any header bytes 
+        /* VDR writes A52 streams without any header bytes
          * (see ftp://ftp.mplayerhq.hu/MPlayer/samples/MPEG-VOB/vdr-AC3) */
         if (datalen >= 4) {
           guint hdr = GST_READ_UINT32_BE (data);
@@ -1825,18 +2036,26 @@ gst_flups_demux_data_cb (GstPESFilter * filter, gboolean first,
         }
 
         if (G_LIKELY (stream_type == -1)) {
-          /* new id */
+          /* new id is in the first byte */
           id = data[offset++];
-          /* Number of audio frames in this packet */
-          nframes = data[offset++];
-
-          GST_DEBUG_OBJECT (demux, "private type 0x%02x, %d frames", id,
-              nframes);
-
-          datalen -= 2;
+          datalen--;
 
           /* and remap */
           stream_type = demux->psm[id];
+
+          /* Now, if it's a subpicture stream - no more, otherwise
+           * take the first byte too, since it's the frame count in audio
+           * streams and our backwards compat convention is to strip it off */
+          if (stream_type != ST_PS_DVD_SUBPICTURE) {
+            /* Number of audio frames in this packet */
+            nframes = data[offset++];
+            datalen--;
+            GST_DEBUG_OBJECT (demux, "private type 0x%02x, %d frames", id,
+                nframes);
+          } else {
+            GST_DEBUG_OBJECT (demux, "private type 0x%02x, stream type %d", id,
+                stream_type);
+          }
         }
       }
       if (stream_type == -1)
@@ -1860,7 +2079,7 @@ gst_flups_demux_data_cb (GstPESFilter * filter, gboolean first,
     demux->current_stream = gst_flups_demux_get_stream (demux, id, stream_type);
   }
 
-  if (demux->current_stream == NULL) {
+  if (G_UNLIKELY (demux->current_stream == NULL)) {
     GST_DEBUG_OBJECT (demux, "Dropping buffer for unknown stream id 0x%02x",
         id);
     goto done;
@@ -1916,7 +2135,7 @@ gst_flups_demux_resync (GstFluPSDemux * demux, gboolean save)
   gboolean found;
 
   avail = gst_adapter_available (demux->adapter);
-  if (avail < 4)
+  if (G_UNLIKELY (avail < 4))
     goto need_data;
 
   /* Common case, read 4 bytes an check it */
@@ -1932,7 +2151,7 @@ gst_flups_demux_resync (GstFluPSDemux * demux, gboolean save)
     return TRUE;
   }
 
-  /* Otherwise, we are starting at byte 4 and we need to search 
+  /* Otherwise, we are starting at byte 4 and we need to search
      the sync code in all available data in the adapter */
   offset = 4;
   if (offset >= avail)
@@ -2367,12 +2586,10 @@ gst_flups_sink_get_duration (GstFluPSDemux * demux)
     gst_segment_set_last_stop (&demux->src_segment, GST_FORMAT_TIME,
         demux->src_segment.start);
   }
-#if POST_10_10
   GST_INFO_OBJECT (demux, "sink segment configured %" GST_SEGMENT_FORMAT,
       &demux->sink_segment);
   GST_INFO_OBJECT (demux, "src segment configured %" GST_SEGMENT_FORMAT,
       &demux->src_segment);
-#endif
 
   res = TRUE;
 
@@ -2410,10 +2627,15 @@ gst_flups_demux_loop (GstPad * pad)
 {
   GstFluPSDemux *demux;
   GstFlowReturn ret = GST_FLOW_OK;
-  guint offset = 0;
+  guint64 offset = 0;
 
   demux = GST_FLUPS_DEMUX (gst_pad_get_parent (pad));
 
+  if (G_UNLIKELY (demux->flushing)) {
+    ret = GST_FLOW_WRONG_STATE;
+    goto pause;
+  }
+
   if (G_UNLIKELY (demux->sink_segment.format == GST_FORMAT_UNDEFINED))
     gst_flups_sink_get_duration (demux);
 
@@ -2449,7 +2671,7 @@ gst_flups_demux_loop (GstPad * pad)
       goto pause;
     }
   } else {                      /* Reverse playback */
-    guint size = MIN (offset, BLOCK_SZ);
+    guint64 size = MIN (offset, BLOCK_SZ);
 
     /* pull in data */
     ret = gst_flups_demux_pull_block (pad, demux, offset - size, size);
@@ -2515,10 +2737,11 @@ pause:
           /* normal playback, send EOS to all linked pads */
           gst_element_no_more_pads (GST_ELEMENT (demux));
           GST_LOG_OBJECT (demux, "Sending EOS, at end of stream");
-          if (!gst_flups_demux_send_event (demux, gst_event_new_eos ())) {
-            GST_WARNING_OBJECT (demux, "failed pushing EOS on streams");
+          if (!gst_flups_demux_send_event (demux, gst_event_new_eos ())
+              && !have_open_streams (demux)) {
+            GST_WARNING_OBJECT (demux, "EOS and no streams open");
             GST_ELEMENT_ERROR (demux, STREAM, FAILED,
-                ("Internal data stream error."), ("Can't push EOS downstream"));
+                ("Internal data stream error."), ("No valid streams detected"));
           }
         }
       } else {
@@ -2661,6 +2884,10 @@ gst_flups_demux_chain (GstPad * pad, GstBuffer * buffer)
   save = TRUE;
   while (gst_flups_demux_resync (demux, save)) {
     gboolean ps_sync = TRUE;
+    if (G_UNLIKELY (demux->flushing)) {
+      ret = GST_FLOW_WRONG_STATE;
+      goto done;
+    }
 
     /* now switch on last synced byte */
     switch (demux->last_sync_code) {
