@@ -1098,8 +1098,8 @@ atom_tag_new (guint32 fourcc, guint32 flags_as_uint)
   tag->header.type = fourcc;
   atom_tag_data_init (&tag->data);
   tag->data.header.flags[2] = flags_as_uint & 0xFF;
-  tag->data.header.flags[1] = flags_as_uint & 0xFF00;
-  tag->data.header.flags[0] = flags_as_uint & 0xFF0000;
+  tag->data.header.flags[1] = (flags_as_uint & 0xFF00) >> 8;
+  tag->data.header.flags[0] = (flags_as_uint & 0xFF0000) >> 16;
   return tag;
 }
 
@@ -2425,9 +2425,13 @@ atom_trak_update_duration (AtomTRAK * trak, guint64 moov_timescale)
 {
   trak->mdia.mdhd.time_info.duration =
       atom_stts_get_total_duration (&trak->mdia.minf.stbl.stts);
-  trak->tkhd.duration =
-      gst_util_uint64_scale (trak->mdia.mdhd.time_info.duration, moov_timescale,
-      trak->mdia.mdhd.time_info.timescale);
+  if (trak->mdia.mdhd.time_info.timescale != 0) {
+    trak->tkhd.duration =
+        gst_util_uint64_scale (trak->mdia.mdhd.time_info.duration,
+        moov_timescale, trak->mdia.mdhd.time_info.timescale);
+  } else {
+    trak->tkhd.duration = 0;
+  }
 }
 
 static guint32
