@@ -230,7 +230,7 @@ best_overlap_offset_s16 (GstScaletempo * scaletempo)
     ppc = p->buf_pre_corr;
     ppc += p->samples_overlap - p->samples_per_frame;
     ps += p->samples_overlap - p->samples_per_frame;
-    i = -(p->samples_overlap - p->samples_per_frame);
+    i = -((glong) p->samples_overlap - (glong) p->samples_per_frame);
     do {
       corr += ppc[i + 0] * ps[i + 0];
       corr += ppc[i + 1] * ps[i + 1];
@@ -555,7 +555,7 @@ gst_scaletempo_sink_event (GstBaseTransform * trans, GstEvent * event)
       rate = 1.0;
       //gst_event_unref (event);
       event = gst_event_new_new_segment_full (update, rate, applied_rate,
-          format, start, stop, position);
+          format, start, (stop - start) / applied_rate + start, position);
       gst_pad_push_event (GST_BASE_TRANSFORM_SRC_PAD (trans), event);
       return FALSE;
     }
