@@ -28,13 +28,15 @@
 
 #include "windows.h"
 
+#define WM_GRAPH_NOTIFY WM_APP + 1 /* Private message */
+
 static const GstElementDetails gst_dshowvideosink_details =
 GST_ELEMENT_DETAILS ("DirectShow video sink",
     "Sink/Video",
     "Display data using a DirectShow video renderer",
     "Pioneers of the Inevitable <songbird@songbirdnest.com>");
 
-GST_DEBUG_CATEGORY_STATIC (dshowvideosink_debug);
+GST_DEBUG_CATEGORY (dshowvideosink_debug);
 #define GST_CAT_DEFAULT dshowvideosink_debug
 
 static GstCaps * gst_directshow_media_type_to_caps (AM_MEDIA_TYPE *mediatype);
@@ -198,6 +200,7 @@ gst_dshowvideosink_clear (GstDshowVideoSink *sink)
   sink->renderersupport = NULL;
   sink->fakesrc = NULL;
   sink->filter_graph = NULL;
+  sink->filter_media_event = NULL;
 
   sink->keep_aspect_ratio = FALSE;
   sink->full_screen = FALSE;
@@ -396,6 +399,20 @@ gst_dshow_get_pin_from_filter (IBaseFilter *filter, PIN_DIRECTION pindir, IPin *
   return ret;
 }
 
+static void 
+gst_dshowvideosink_handle_event (GstDshowVideoSink *sink)
+{
+  if (sink->filter_media_event) {
+    long evCode;
+    LONG_PTR param1, param2;
+    while (SUCCEEDED (sink->filter_media_event->GetEvent(&evCode, &param1, &param2, 0)))
+    {
+      GST_INFO_OBJECT (sink, "Received DirectShow graph event code 0x%x", evCode);
+      sink->filter_media_event->FreeEventParams(evCode, param1, param2);
+    }
+  }
+}
+
 /* WNDPROC for application-supplied windows */
 LRESULT APIENTRY WndProcHook (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
@@ -405,6 +422,9 @@ LRESULT APIENTRY WndProcHook (HWND hWnd, UINT message, WPARAM wParam, LPARAM lPa
   GstDshowVideoSink *sink = (GstDshowVideoSink *)GetProp (hWnd, L"GstDShowVideoSink");
 
   switch (message) {
+    case WM_GRAPH_NOTIFY:
+      gst_dshowvideosink_handle_event (sink);
+      return 0;
     case WM_PAINT:
       sink->renderersupport->PaintWindow ();
       break;
@@ -436,9 +456,13 @@ WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
     return DefWindowProc (hWnd, message, wParam, lParam);
   }
 
-  GST_DEBUG_OBJECT (sink, "Got a window message for %x, %x", hWnd, message);
+  //GST_DEBUG_OBJECT (sink, "Got a window message for %x, %x", hWnd, message);
 
   switch (message) {
+    case WM_GRAPH_NOTIFY:
+      GST_LOG_OBJECT (sink, "GRAPH_NOTIFY WINDOW MESSAGE");
+      gst_dshowvideosink_handle_event (sink);
+      return 0;
     case WM_PAINT:
       sink->renderersupport->PaintWindow ();
       break;
@@ -538,6 +562,8 @@ gst_dshowvideosink_window_thread (GstDshowVideoSink * sink)
 
   SetWindowLongPtr (video_window, GWLP_USERDATA, (LONG)sink);
 
+  sink->window_id = video_window;
+
   /* signal application we created a window */
   gst_x_overlay_got_xwindow_id (GST_X_OVERLAY (sink),
       (gulong)video_window);
@@ -637,6 +663,8 @@ static void gst_dshowvideosink_set_window_for_renderer (GstDshowVideoSink *sink)
 static void
 gst_dshowvideosink_prepare_window (GstDshowVideoSink *sink)
 {
+  HRESULT hres;
+
   /* Give the app a last chance to supply a window id */
   if (!sink->window_id) {
     gst_x_overlay_prepare_xwindow_id (GST_X_OVERLAY (sink));
@@ -650,6 +678,23 @@ gst_dshowvideosink_prepare_window (GstDshowVideoSink *sink)
   else {
     gst_dshowvideosink_create_default_window (sink);
   }
+
+  if (sink->filter_media_event) {
+    sink->filter_media_event->Release();
+    sink->filter_media_event = NULL;
+  }
+
+  hres = sink->filter_graph->QueryInterface(
+          IID_IMediaEventEx, (void **) &sink->filter_media_event);
+
+  if (FAILED (hres)) {
+    GST_WARNING_OBJECT (sink, "Failed to get IMediaEventEx");
+  }
+  else {
+    hres = sink->filter_media_event->SetNotifyWindow ((OAHWND)sink->window_id,
+            WM_GRAPH_NOTIFY, 0);
+    GST_DEBUG_OBJECT (sink, "SetNotifyWindow(%p) returned %x", sink->window_id, hres);
+  }
 }
 
 static gboolean
@@ -823,7 +868,7 @@ static GstStateChangeReturn
 gst_dshowvideosink_change_state (GstElement * element, GstStateChange transition)
 {
   GstDshowVideoSink *sink = GST_DSHOWVIDEOSINK (element);
-  GstStateChangeReturn ret;
+  GstStateChangeReturn ret, rettmp;
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
@@ -832,7 +877,7 @@ gst_dshowvideosink_change_state (GstElement * element, GstStateChange transition
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       ret = gst_dshowvideosink_start_graph (sink);
-      if (ret != GST_STATE_CHANGE_SUCCESS)
+      if (ret == GST_STATE_CHANGE_FAILURE)
         return ret;
       break;
   }
@@ -841,10 +886,14 @@ gst_dshowvideosink_change_state (GstElement * element, GstStateChange transition
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      ret = gst_dshowvideosink_pause_graph (sink);
+      rettmp = gst_dshowvideosink_pause_graph (sink);
+      if (rettmp == GST_STATE_CHANGE_FAILURE)
+        ret = rettmp;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      ret = gst_dshowvideosink_stop_graph (sink);
+      rettmp = gst_dshowvideosink_stop_graph (sink);
+      if (rettmp == GST_STATE_CHANGE_FAILURE)
+        ret = rettmp;
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       gst_dshowvideosink_clear (sink);
@@ -1152,6 +1201,11 @@ static gboolean
 gst_dshowvideosink_build_filtergraph (GstDshowVideoSink *sink)
 {
   HRESULT hres;
+  gboolean comInit = FALSE;
+  
+  hres = CoInitialize(0);
+  if (SUCCEEDED (hres))
+    comInit = TRUE;
 
   /* Build our DirectShow FilterGraph, looking like: 
    *
@@ -1203,6 +1257,8 @@ gst_dshowvideosink_build_filtergraph (GstDshowVideoSink *sink)
     goto error;
   }
 
+  if (comInit)
+    CoUninitialize();
   return TRUE;
 
 error:
@@ -1216,6 +1272,14 @@ error:
     sink->filter_graph = NULL;
   }
 
+  if (sink->filter_media_event) {
+    sink->filter_media_event->Release();
+    sink->filter_media_event = NULL;
+  }
+
+  if (comInit)
+    CoUninitialize();
+
   return FALSE;
 }
 
@@ -1234,18 +1298,24 @@ gst_dshowvideosink_set_caps (GstBaseSink * bsink, GstCaps * caps)
 {
   GstDshowVideoSink *sink = GST_DSHOWVIDEOSINK (bsink);
 
-  /* TODO: What do we want to do if the caps change while we're running?
-   * Find out if we can handle this or not... */
+  if (sink->connected) {
+    /* Look at the DShow APIs for dynamically modifying the pipeline and see
+     * if we can make this work later... */
+    GST_WARNING_OBJECT (sink, "Changing caps at runtime is not yet supported");
+    return FALSE;
+  }
 
   if (!gst_caps_to_directshow_media_type (caps, &sink->mediatype)) {
     GST_WARNING_OBJECT (sink, "Cannot convert caps to AM_MEDIA_TYPE, rejecting");
     return FALSE;
   }
 
+  GST_DEBUG_OBJECT (sink, "Configuring output pin media type");
   /* Now we have an AM_MEDIA_TYPE describing what we're going to send.
    * We set this on our DirectShow fakesrc's output pin. 
    */
   sink->fakesrc->GetOutputPin()->SetMediaType (&sink->mediatype);
+  GST_DEBUG_OBJECT (sink, "Configured output pin media type");
 
   return TRUE;
 }
@@ -1296,7 +1366,7 @@ gst_dshowvideosink_render (GstBaseSink *bsink, GstBuffer *buffer)
 
   GST_DEBUG_OBJECT (sink, "Pushing buffer through fakesrc->renderer");
   ret = sink->fakesrc->GetOutputPin()->PushBuffer (buffer);
-  GST_DEBUG_OBJECT (sink, "Done pushing buffer through fakesrc->renderer");
+  GST_DEBUG_OBJECT (sink, "Done pushing buffer through fakesrc->renderer: %s", gst_flow_get_name(ret));
 
   return ret;
 }
