@@ -41,6 +41,7 @@
 #include <string.h>
 
 #include "mpegutil.h"
+#include "gstvdputils.h"
 #include "gstvdpmpegdec.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_vdp_mpeg_dec_debug);
@@ -55,8 +56,7 @@ enum
 
 enum
 {
-  PROP_0,
-  PROP_DISPLAY
+  PROP_0
 };
 
 /* the capabilities of the inputs and outputs.
@@ -82,79 +82,55 @@ GST_BOILERPLATE_FULL (GstVdpMpegDec, gst_vdp_mpeg_dec,
 
 static void gst_vdp_mpeg_dec_init_info (VdpPictureInfoMPEG1Or2 * vdp_info);
 static void gst_vdp_mpeg_dec_finalize (GObject * object);
-static void gst_vdp_mpeg_dec_set_property (GObject * object,
-    guint prop_id, const GValue * value, GParamSpec * pspec);
-static void gst_vdp_mpeg_dec_get_property (GObject * object,
-    guint prop_id, GValue * value, GParamSpec * pspec);
-
-guint8 *
-mpeg_util_find_start_code (guint32 * sync_word, guint8 * cur, guint8 * end)
-{
-  guint32 code;
-
-  if (G_UNLIKELY (cur == NULL))
-    return NULL;
-
-  code = *sync_word;
-
-  while (cur < end) {
-    code <<= 8;
-
-    if (code == 0x00000100) {
-      /* Reset the sync word accumulator */
-      *sync_word = 0xffffffff;
-      return cur;
-    }
-
-    /* Add the next available byte to the collected sync word */
-    code |= *cur++;
-  }
-
-  *sync_word = code;
-  return NULL;
-}
 
 typedef struct
 {
+  GstByteReader reader;
   GstBuffer *buffer;
-  guint8 *cur;
-  guint8 *end;
+  guint start;
 } GstVdpMpegPacketizer;
 
 static GstBuffer *
 gst_vdp_mpeg_packetizer_get_next_packet (GstVdpMpegPacketizer * packetizer)
 {
-  guint32 sync_word = 0xffffff;
-  guint8 *packet_start;
-  guint8 *packet_end;
+  guint offset, size;
+  GstBuffer *buf;
 
-  if (!packetizer->cur)
+  if (packetizer->start == -1)
     return NULL;
 
-  packet_start = packetizer->cur - 3;
-  packetizer->cur = packet_end = mpeg_util_find_start_code (&sync_word,
-      packetizer->cur, packetizer->end);
+  if (!gst_byte_reader_set_pos (&packetizer->reader, packetizer->start + 3))
+    return NULL;
 
-  if (packet_end)
-    packet_end -= 3;
-  else
-    packet_end = packetizer->end;
+  offset = gst_byte_reader_masked_scan_uint32 (&packetizer->reader, 0xffffff00,
+      0x00000100, 0, gst_byte_reader_get_remaining (&packetizer->reader));
 
-  return gst_buffer_create_sub (packetizer->buffer,
-      packet_start - GST_BUFFER_DATA (packetizer->buffer),
-      packet_end - packet_start);
+  if (offset != -1) {
+    offset = gst_byte_reader_get_pos (&packetizer->reader) + offset;
+    size = offset - packetizer->start;
+  } else
+    size = gst_byte_reader_get_remaining (&packetizer->reader) + 3;
+
+  buf = gst_buffer_create_sub (packetizer->buffer, packetizer->start, size);
+
+  packetizer->start = offset;
+
+  return buf;
 }
 
 static void
 gst_vdp_mpeg_packetizer_init (GstVdpMpegPacketizer * packetizer,
     GstBuffer * buffer)
 {
-  guint32 sync_word = 0xffffffff;
+  guint offset;
 
+  gst_byte_reader_init_from_buffer (&packetizer->reader, buffer);
   packetizer->buffer = buffer;
-  packetizer->end = GST_BUFFER_DATA (buffer) + GST_BUFFER_SIZE (buffer);
-  packetizer->cur = mpeg_util_find_start_code (&sync_word,
-      GST_BUFFER_DATA (buffer), packetizer->end);
+
+  offset = gst_byte_reader_masked_scan_uint32 (&packetizer->reader, 0xffffff00,
+      0x00000100, 0, gst_byte_reader_get_remaining (&packetizer->reader));
+
+  packetizer->start = offset;
 }
 
 static gboolean
@@ -172,10 +148,6 @@ gst_vdp_mpeg_dec_set_caps (GstPad * pad, GstCaps * caps)
   gboolean res;
 
   const GValue *value;
-  /* Default to MPEG1 until we find otherwise */
-  VdpDecoderProfile profile = VDP_DECODER_PROFILE_MPEG1;
-  GstVdpDevice *device;
-  VdpStatus status;
 
   structure = gst_caps_get_structure (caps, 0);
 
@@ -187,7 +159,6 @@ gst_vdp_mpeg_dec_set_caps (GstPad * pad, GstCaps * caps)
   gst_structure_get_boolean (structure, "interlaced", &interlaced);
 
   src_caps = gst_caps_new_simple ("video/x-vdpau-video",
-      "device", G_TYPE_OBJECT, mpeg_dec->device,
       "chroma-type", G_TYPE_INT, VDP_CHROMA_TYPE_420,
       "width", G_TYPE_INT, width,
       "height", G_TYPE_INT, height,
@@ -212,6 +183,9 @@ gst_vdp_mpeg_dec_set_caps (GstPad * pad, GstCaps * caps)
   /* parse caps to setup decoder */
   gst_structure_get_int (structure, "mpegversion", &mpeg_dec->version);
 
+  /* Default to MPEG1 until we find otherwise */
+  mpeg_dec->profile = VDP_DECODER_PROFILE_MPEG1;
+
   value = gst_structure_get_value (structure, "codec_data");
   if (value) {
     GstBuffer *codec_data, *buf;
@@ -240,10 +214,10 @@ gst_vdp_mpeg_dec_set_caps (GstPad * pad, GstCaps * caps)
         if (mpeg_dec->version != 1) {
           switch (ext.profile) {
             case 5:
-              profile = VDP_DECODER_PROFILE_MPEG2_SIMPLE;
+              mpeg_dec->profile = VDP_DECODER_PROFILE_MPEG2_SIMPLE;
               break;
             default:
-              profile = VDP_DECODER_PROFILE_MPEG2_MAIN;
+              mpeg_dec->profile = VDP_DECODER_PROFILE_MPEG2_MAIN;
               break;
           }
         }
@@ -261,23 +235,6 @@ gst_vdp_mpeg_dec_set_caps (GstPad * pad, GstCaps * caps)
     }
   }
 
-  device = mpeg_dec->device;
-
-  if (mpeg_dec->decoder != VDP_INVALID_HANDLE) {
-    device->vdp_decoder_destroy (mpeg_dec->decoder);
-    mpeg_dec->decoder = VDP_INVALID_HANDLE;
-  }
-
-  status = device->vdp_decoder_create (device->device, profile, mpeg_dec->width,
-      mpeg_dec->height, 2, &mpeg_dec->decoder);
-  if (status != VDP_STATUS_OK) {
-    GST_ELEMENT_ERROR (mpeg_dec, RESOURCE, READ,
-        ("Could not create vdpau decoder"),
-        ("Error returned from vdpau was: %s",
-            device->vdp_get_error_string (status)));
-    res = FALSE;
-    goto done;
-  }
   res = TRUE;
 
 done:
@@ -337,13 +294,44 @@ gst_vdp_mpeg_dec_push_video_buffer (GstVdpMpegDec * mpeg_dec,
   return gst_pad_push (mpeg_dec->src, GST_BUFFER (buffer));
 }
 
+static GstFlowReturn
+gst_vdp_mpeg_dec_alloc_buffer (GstVdpMpegDec * mpeg_dec, GstBuffer ** outbuf)
+{
+  GstFlowReturn ret;
+
+  ret = gst_pad_alloc_buffer_and_set_caps (mpeg_dec->src, 0, 0,
+      GST_PAD_CAPS (mpeg_dec->src), outbuf);
+  if (ret != GST_FLOW_OK)
+    return ret;
+
+  if (!mpeg_dec->device) {
+    GstVdpDevice *device;
+    VdpStatus status;
+
+    device = mpeg_dec->device =
+        g_object_ref (GST_VDP_VIDEO_BUFFER (*outbuf)->device);
+
+    status = device->vdp_decoder_create (device->device, mpeg_dec->profile,
+        mpeg_dec->width, mpeg_dec->height, 2, &mpeg_dec->decoder);
+    if (status != VDP_STATUS_OK) {
+      GST_ELEMENT_ERROR (mpeg_dec, RESOURCE, READ,
+          ("Could not create vdpau decoder"),
+          ("Error returned from vdpau was: %s",
+              device->vdp_get_error_string (status)));
+      ret = GST_FLOW_ERROR;
+    }
+  }
+
+  return ret;
+}
+
 static GstFlowReturn
 gst_vdp_mpeg_dec_decode (GstVdpMpegDec * mpeg_dec,
     GstClockTime timestamp, gint64 size)
 {
   VdpPictureInfoMPEG1Or2 *info;
   GstBuffer *buffer;
-  GstVdpVideoBuffer *outbuf;
+  GstBuffer *outbuf;
   VdpVideoSurface surface;
   GstVdpDevice *device;
   VdpBitstreamBuffer vbit[1];
@@ -351,27 +339,6 @@ gst_vdp_mpeg_dec_decode (GstVdpMpegDec * mpeg_dec,
 
   info = &mpeg_dec->vdp_info;
 
-  buffer = gst_adapter_take_buffer (mpeg_dec->adapter,
-      gst_adapter_available (mpeg_dec->adapter));
-
-  outbuf = gst_vdp_video_buffer_new (mpeg_dec->device, VDP_CHROMA_TYPE_420,
-      mpeg_dec->width, mpeg_dec->height);
-  GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
-  GST_BUFFER_DURATION (outbuf) = mpeg_dec->duration;
-  GST_BUFFER_OFFSET (outbuf) = mpeg_dec->frame_nr;
-  GST_BUFFER_SIZE (outbuf) = size;
-
-  if (info->picture_coding_type == I_FRAME)
-    GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
-  else
-    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
-
-  if (info->top_field_first)
-    GST_BUFFER_FLAG_SET (outbuf, GST_VIDEO_BUFFER_TFF);
-  else
-    GST_BUFFER_FLAG_UNSET (outbuf, GST_VIDEO_BUFFER_TFF);
-
-
   if (info->picture_coding_type != B_FRAME) {
     if (info->backward_reference != VDP_INVALID_HANDLE) {
       gst_buffer_ref (mpeg_dec->b_buffer);
@@ -390,19 +357,42 @@ gst_vdp_mpeg_dec_decode (GstVdpMpegDec * mpeg_dec,
     info->backward_reference = VDP_INVALID_HANDLE;
   }
 
+  if (gst_vdp_mpeg_dec_alloc_buffer (mpeg_dec, &outbuf) != GST_FLOW_OK) {
+    gst_adapter_clear (mpeg_dec->adapter);
+    return GST_FLOW_ERROR;
+  }
+
+  device = GST_VDP_VIDEO_BUFFER (outbuf)->device;
+
   if (info->forward_reference != VDP_INVALID_HANDLE &&
       info->picture_coding_type != I_FRAME)
-    gst_vdp_video_buffer_add_reference (outbuf,
+    gst_vdp_video_buffer_add_reference (GST_VDP_VIDEO_BUFFER (outbuf),
         GST_VDP_VIDEO_BUFFER (mpeg_dec->f_buffer));
 
   if (info->backward_reference != VDP_INVALID_HANDLE
       && info->picture_coding_type == B_FRAME)
-    gst_vdp_video_buffer_add_reference (outbuf,
+    gst_vdp_video_buffer_add_reference (GST_VDP_VIDEO_BUFFER (outbuf),
         GST_VDP_VIDEO_BUFFER (mpeg_dec->b_buffer));
 
-  surface = outbuf->surface;
+  GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
+  GST_BUFFER_DURATION (outbuf) = mpeg_dec->duration;
+  GST_BUFFER_OFFSET (outbuf) = mpeg_dec->frame_nr;
+  GST_BUFFER_SIZE (outbuf) = size;
 
-  device = mpeg_dec->device;
+  if (info->picture_coding_type == I_FRAME)
+    GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
+  else
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
+
+  if (info->top_field_first)
+    GST_BUFFER_FLAG_SET (outbuf, GST_VIDEO_BUFFER_TFF);
+  else
+    GST_BUFFER_FLAG_UNSET (outbuf, GST_VIDEO_BUFFER_TFF);
+
+  buffer = gst_adapter_take_buffer (mpeg_dec->adapter,
+      gst_adapter_available (mpeg_dec->adapter));
+
+  surface = GST_VDP_VIDEO_BUFFER (outbuf)->surface;
 
   vbit[0].struct_version = VDP_BITSTREAM_BUFFER_VERSION;
   vbit[0].bitstream = GST_BUFFER_DATA (buffer);
@@ -589,24 +579,19 @@ gst_vdp_mpeg_dec_flush (GstVdpMpegDec * mpeg_dec)
   if (mpeg_dec->vdp_info.backward_reference != VDP_INVALID_HANDLE)
     gst_buffer_unref (mpeg_dec->b_buffer);
 
-  gst_vdp_mpeg_dec_init_info (&mpeg_dec->vdp_info);
-
   gst_adapter_clear (mpeg_dec->adapter);
 
+  gst_vdp_mpeg_dec_init_info (&mpeg_dec->vdp_info);
   mpeg_dec->next_timestamp = GST_CLOCK_TIME_NONE;
 }
 
 static void
-gst_vdp_mpeg_dec_reset (GstVdpMpegDec * mpeg_dec)
+gst_vdp_mpeg_dec_start (GstVdpMpegDec * mpeg_dec)
 {
-  gst_vdp_mpeg_dec_flush (mpeg_dec);
+  gst_vdp_mpeg_dec_init_info (&mpeg_dec->vdp_info);
 
-  if (mpeg_dec->decoder != VDP_INVALID_HANDLE)
-    mpeg_dec->device->vdp_decoder_destroy (mpeg_dec->decoder);
-  mpeg_dec->decoder = VDP_INVALID_HANDLE;
-  if (mpeg_dec->device)
-    g_object_unref (mpeg_dec->device);
   mpeg_dec->device = NULL;
+  mpeg_dec->decoder = VDP_INVALID_HANDLE;
 
   mpeg_dec->state = GST_VDP_MPEG_DEC_NEED_SEQUENCE;
 
@@ -617,6 +602,24 @@ gst_vdp_mpeg_dec_reset (GstVdpMpegDec * mpeg_dec)
   mpeg_dec->accumulated_duration = 0;
 }
 
+static void
+gst_vdp_mpeg_dec_stop (GstVdpMpegDec * mpeg_dec)
+{
+  if (mpeg_dec->device) {
+    if (mpeg_dec->decoder != VDP_INVALID_HANDLE)
+      mpeg_dec->device->vdp_decoder_destroy (mpeg_dec->decoder);
+
+    g_object_unref (mpeg_dec->device);
+  }
+
+  if (mpeg_dec->vdp_info.forward_reference != VDP_INVALID_HANDLE)
+    mpeg_dec->vdp_info.forward_reference = VDP_INVALID_HANDLE;
+  if (mpeg_dec->vdp_info.backward_reference != VDP_INVALID_HANDLE)
+    mpeg_dec->vdp_info.backward_reference = VDP_INVALID_HANDLE;
+
+  gst_adapter_clear (mpeg_dec->adapter);
+}
+
 static GstFlowReturn
 gst_vdp_mpeg_dec_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -1000,7 +1003,7 @@ gst_vdp_mpeg_dec_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      mpeg_dec->device = gst_vdp_get_device (mpeg_dec->display_name);
+      gst_vdp_mpeg_dec_start (mpeg_dec);
       break;
     default:
       break;
@@ -1010,7 +1013,7 @@ gst_vdp_mpeg_dec_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_vdp_mpeg_dec_reset (mpeg_dec);
+      gst_vdp_mpeg_dec_stop (mpeg_dec);
       break;
     default:
       break;
@@ -1049,15 +1052,8 @@ gst_vdp_mpeg_dec_class_init (GstVdpMpegDecClass * klass)
   gstelement_class = (GstElementClass *) klass;
 
   gobject_class->finalize = gst_vdp_mpeg_dec_finalize;
-  gobject_class->set_property = gst_vdp_mpeg_dec_set_property;
-  gobject_class->get_property = gst_vdp_mpeg_dec_get_property;
 
-  gstelement_class->change_state =
-      GST_DEBUG_FUNCPTR (gst_vdp_mpeg_dec_change_state);
-
-  g_object_class_install_property (gobject_class, PROP_DISPLAY,
-      g_param_spec_string ("display", "Display", "X Display name",
-          NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+  gstelement_class->change_state = gst_vdp_mpeg_dec_change_state;
 }
 
 static void
@@ -1099,16 +1095,7 @@ gst_vdp_mpeg_dec_init (GstVdpMpegDec * mpeg_dec, GstVdpMpegDecClass * gclass)
       GST_DEBUG_FUNCPTR (gst_vdp_mpeg_dec_sink_event));
   gst_element_add_pad (GST_ELEMENT (mpeg_dec), mpeg_dec->sink);
 
-  mpeg_dec->display_name = NULL;
   mpeg_dec->adapter = gst_adapter_new ();
-
-  mpeg_dec->device = NULL;
-  mpeg_dec->decoder = VDP_INVALID_HANDLE;
-  mpeg_dec->vdp_info.forward_reference = VDP_INVALID_HANDLE;
-  mpeg_dec->vdp_info.backward_reference = VDP_INVALID_HANDLE;
-
-  gst_vdp_mpeg_dec_reset (mpeg_dec);
-
   mpeg_dec->mutex = g_mutex_new ();
 }
 
@@ -1120,36 +1107,3 @@ gst_vdp_mpeg_dec_finalize (GObject * object)
   g_object_unref (mpeg_dec->adapter);
   g_mutex_free (mpeg_dec->mutex);
 }
-
-static void
-gst_vdp_mpeg_dec_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstVdpMpegDec *mpeg_dec = GST_VDP_MPEG_DEC (object);
-
-  switch (prop_id) {
-    case PROP_DISPLAY:
-      g_free (mpeg_dec->display_name);
-      mpeg_dec->display_name = g_value_dup_string (value);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_vdp_mpeg_dec_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstVdpMpegDec *mpeg_dec = GST_VDP_MPEG_DEC (object);
-
-  switch (prop_id) {
-    case PROP_DISPLAY:
-      g_value_set_string (value, mpeg_dec->display_name);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
